#!/opt/local/bin/python
#
# draw MHD plots

import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")


import locale
import numpy              as np
import matplotlib.pyplot  as plt
from read_mhd                import *
from pkg_mhdplot             import mhd_vpkg
from openggcm_info           import runinfo,mhdtime,mhdt2tstr
from matplotlib.colors       import LogNorm
from matplotlib.ticker       import MultipleLocator, FuncFormatter
from mpl_toolkits.axes_grid1 import make_axes_locatable



def mhd_2Dplot(nrow,ncol,fps,pn,lp,xran,yran,xx,yy,val,vstr,title):
  "plot mhd"

####### tick info for pn=xr
  def xr_zgse(x, pos):
    value=x*cos(radians(lp))
    return '%2d' % (round(value))
  def xr_ygse(x, pos):
    value=-x*sin(radians(lp))
    return '%2d' % (round(value))

####### draw graphs
  fig = plt.figure(figsize=(8,11))

  for i in range(len(val)):

    ax1      = fig.add_subplot(nrow,ncol,i+1,aspect='equal')
    ax1.tick_params(axis='both',which='major',labelsize=7)
    x,y      = np.meshgrid(xx,yy)
    pkg_plot = mhd_vpkg(vstr[i])

    if pkg_plot[5]==1:
      cs    = ax1.contourf(x,y,val[i]*pkg_plot[2],levels=pkg_plot[3], \
              cmap=pkg_plot[1],norm=LogNorm())
    else :
#      print(np.shape(x),np.shape(y),np.shape(val[i]),np.shape(pkg_plot[2]))
      cs    = ax1.contourf(x,y,val[i]*pkg_plot[2],levels=pkg_plot[3], cmap=pkg_plot[1],extend='both')

    cax1  = make_axes_locatable(ax1)
    cax2  = cax1.append_axes("right",size="3%",pad="5%")
    cb    = fig.colorbar(cs,cax=cax2,ticks=pkg_plot[4])
    cl    = plt.getp(cb.ax, 'ymajorticklabels')
    plt.setp(cl, fontsize=7)


#   circle1=plt.Circle((2.,0),3,facecolor='None',edgecolor='magenta')
#   circle2=plt.Circle((-2.,0),3,facecolor='None',edgecolor='magenta')
#   ax1.add_artist(circle1)
#   ax1.add_artist(circle2)


####### tick infomation

    if pn!='xr':
      ax1.yaxis.set_major_locator(MultipleLocator(yran[2]))
      ax1.yaxis.set_minor_locator(MultipleLocator(yran[3]))
      if pn=='xy' : xl='Xgse [Re]';yl='Ygse [Re]'
      if pn=='xz' : xl='Xgse [Re]';yl='Zgse [Re]'
      if pn=='yz' : xl='Ygse [Re]';yl='Zgse [Re]'
      if pn=='sc' : xl='Spin angle [deg]';yl='Polar angle [deg]'

    if pn=='xr': # xaxis rotation
      xl      = 'Xgse [Re]'
      yl      = 'Zgse [Re]'

    if pn=='zr': # zaxis rotation
      xl      = 'Xgse [Re]'
      yl      = 'Zgse [Re]'
## calculate zgse
      ax1.yaxis.set_major_locator(MultipleLocator(abs(yran[2]/np.cos(np.radians(lp)))))
      ax1.yaxis.set_minor_locator(MultipleLocator(abs(yran[3]/np.cos(np.radians(lp)))))
      ax1.yaxis.set_major_formatter(FuncFormatter(xr_zgse))
## calculate ygse
#     aa1=ax1.twinx()
#     aa1.set_ylim(-yran[0]*np.sin(np.radians(lp)),-yran[1]*np.sin(np.radians(lp)))
#     aa1.yaxis.set_major_locator(MultipleLocator(abs(yran[2]/np.sin(np.radians(lp)))))
#     aa1.yaxis.set_minor_locator(MultipleLocator(abs(yran[3]/np.sin(np.radians(lp)))))
#     aa1.grid(True)
#     aa1.yaxis.set_major_formatter(FuncFormatter(xr_ygse))

    ax1.xaxis.set_major_locator(MultipleLocator(xran[2]))
    ax1.xaxis.set_minor_locator(MultipleLocator(xran[3]))
    ax1.set_xlim(xran[0],xran[1])
    ax1.set_ylim(yran[0],yran[1])

    if i%ncol==0         : ax1.set_ylabel(yl,fontsize=10)
    if i>ncol*(nrow-1)-1 : ax1.set_xlabel(xl,fontsize=10)
    ax1.grid(True)


##### titles for the figures
    if pkg_plot[5]==1:
      vmax,vmin = maxmin_ran([xran[0],xran[1]],[yran[0],yran[1]],xx,yy,np.log10(val[i]*pkg_plot[2]))
    else :
      vmax,vmin = maxmin_ran([xran[0],xran[1]],[yran[0],yran[1]],xx,yy,val[i]*pkg_plot[2])

    lmax = "Max: %.2f "%vmax
    lmin = "Min: %.2f "%vmin
    print (vstr[i],lmax,lmin)

#   if i==0:  ax1.set_title(title,fontsize=10)
#plt.figtext(0.5,0.95,title,fontsize=13,ha='center',va='bottom')
    if i==0: ax1.text(0.0,1.1,title,fontsize=10,ha='left', va='bottom',transform=ax1.transAxes)
    ax1.text(1.0,1.0,pkg_plot[0],fontsize=10,ha='right',va='bottom',transform=ax1.transAxes)
    ax1.text(0.0,0.0,lmin       ,fontsize=10,ha='left' ,va='bottom',transform=ax1.transAxes)
    ax1.text(1.0,0.0,lmax       ,fontsize=10,ha='right',va='bottom',transform=ax1.transAxes)


  plt.savefig(fps)

  return





def mhd_info(Dps,info,fmhd,pn,lp,xran,yran,tash):
  "set information for mhd plots 						\
   read_2DMHD gives MHD values in the following orders				\
   vv   = [ bx,by,bz,bt,vx,vy,vz,vt,ex,ey,ez,et,jx,jy,jz,jt,rr,pp,pdy,kev ]	"


  nrow=4
  ncol=1

  xx,yy,vv,vstr = read_2DMHD(fmhd,pn)


  val1=[ [vv[0],vv[1],vv[2],vv[3]], 	\
         [vv[4],vv[5],vv[6],vv[7]], 	\
         [vv[8],vv[9],vv[10],vv[11]], 	\
         [vv[12],vv[13],vv[14],vv[15]],	\
         [vv[16],vv[17],vv[18],vv[19]], \
         [vv[22],vv[23],vv[24],vv[25]], \
         [vv[20],vv[21]]  		]


  vstr1=[ [vstr[0],vstr[1],vstr[2],vstr[3]], 		\
          [vstr[4],vstr[5],vstr[6],vstr[7]], 		\
          [vstr[8],vstr[9],vstr[10],vstr[11]], 		\
          [vstr[12],vstr[13],vstr[14],vstr[15]], 	\
          [vstr[16],vstr[17],vstr[18],vstr[19]],	\
          [vstr[22],vstr[23],vstr[24],vstr[25]],	\
          [vstr[20],vstr[21]] 				]


  fname=[ 'bxyzt', \
          'vxyzt', \
          'exyzt', \
          'jxyzt', \
          'rppt' , \
          'xray' , \
          'epareta'   ]

# nrow = 1
# ncol = 1
# val1 = [[vv[16]]]
# vstr1= [[vstr[16]]]
# fname= ['rr']

  time = mhdt2tstr(locale.atoi(info.mhdt)+tash,info.ti)
  mhdt = mhdtime(time)

  if tash==0: title= mhdt.name+'\n'+info.run+'.'+pn+'_'+str(lp)+'.'+info.mhdt
  else: title= mhdt.name+': %dmin shifted\n'%int(tash/60)+info.run+'.'+pn+'_'+str(lp)+'.'+info.mhdt

  print ('============================================================================')
  lps=''
  for i in [0,1,4,5] :

    Dps1=Dps+'/'+fname[i]
    if not os.path.exists(Dps1): os.system('mkdir '+Dps1)

    fps = Dps1+'/'+info.run+'.'+fname[i]+'.'+info.mhdt+'.'+pn+'_'+str(lp)+'.png'
    print ("SAVE ",fps)

    mhd_2Dplot(nrow,ncol,fps,pn,lp,xran,yran,xx,yy,val1[i],vstr1[i],title)

    lps=lps+' '+fps


  print ('============================================================================')
  print ("MAKE ONE SUMMARY PLOT")

  Dps1=Dps+'/summary/'
  if not os.path.exists(Dps1): os.system('mkdir '+Dps1)
  fname=Dps1+info.run+'.'+pn+'_'+str(lp)+'.'+info.mhdt+'.jpg'
  cmd='convert -trim '+lps+' +append '+fname
  print (cmd)
  os.system(cmd)

  return


def main(argv):

  fname = argv[0].split(':') 	        # list of filenames
  fgrd  = fname[0]
  f3df  = fname[1]
  Dmhd  = fname[2]			# directory of openggcm ascii file
#  print(Dmhd)
  Dps   = fname[3]                      # directory to save plots


  vran  = argv[1].split(':')            # value range: pn:lp:xi:xf:yi:yf:ds
  pn    = vran[0]                       # plane : xy,xz,yz,xr
  lp    = locale.atof(vran[1])          # plane location
  xi    = locale.atof(vran[2])
  xf    = locale.atof(vran[3])
  yi    = locale.atof(vran[4])
  yf    = locale.atof(vran[5])
  ds    = locale.atof(vran[6])


  pran  = argv[2].split(':')         	# plot range: x1:x2:dx1:dx2:y1:y2:dy1:dy2:re
  x1    = locale.atof(pran[0])		# xrange x1-x2
  x2    = locale.atof(pran[1])
  dx1   = locale.atof(pran[2])		# major x-tick interval
  dx2   = locale.atof(pran[3])		# minor x-tick interval
  y1    = locale.atof(pran[4])		# yrange y1-y2
  y2    = locale.atof(pran[5])
  dy1   = locale.atof(pran[6])		# major y-tick interval
  dy2   = locale.atof(pran[7])		# minor y-tick interval
  xran  = [x1,x2,dx1,dx2]
  yran  = [y1,y2,dy1,dy2]

  tash  = locale.atoi(argv[3])

  info  = runinfo(f3df)
  fmhd  = Dmhd+'/'+info.run+'.'+pn+'_'+vran[1]+'.'+info.mhdt
  #print('fmhd',fmhd)
  Dps1  = Dps+'/'+pn+'_'+vran[1]
  #print(Dps1)
  if not os.path.exists(Dps1): os.system('mkdir '+Dps1)

###### create and read an ascii file of openggcm
# if os.path.isfile(fmhd):
#   print '============================================================================'
#   print "create ",fmhd
#   print ' '
#   test=raw_input('Fmhd exists! Press (o/O) to Overwrite and (u/U) to use the old Fmhd :')
#   if   test=='o' or test=='O' :
#      os.system('rm '+fmhd)
#      create_mhd(fmhd,fgrd,f3df,info.dipt,pn,lp,xi,xf,yi,yf,ds)
#   elif test=='u' or test=='U' :
#      pass
#   else           :
#      print "Error! Press O for Overwrite, U for Using old file. Please Run again"
#      sys.exit(1)
# else :
#      create_mhd(fmhd,fgrd,f3df,info.dipt,pn,lp,xi,xf,yi,yf,ds)

#  print('mhdinfo('+str(Dps1)+','+str(info)+','+str(fmhd)+','+str(pn)+','+str(lp)+','+str(xran)+','+str(yran)+','+str(tash)+')')
  mhd_info(Dps1,info,fmhd,pn,lp,xran,yran,tash)

####### one plot per a figure
# xx,yy,vv,vstr = read_2DMHD(fmhd,pn)
# print '============================================================================'
# for i in range(len(vstr)):
#   fps = Dps+'/'+vstr[i]+'/'+info.run+'.'+vstr[i]+'.'+info.mhdt+'.'+pn+'_'+vran[1]+'.ps'
#   print "SAVE ",fps
#   mhd_2Dplots(1,1,fps,pn,lp,xran,yran,xx,yy,[vv[i]],[vstr[i]])
####### several plots per a figure

  return


# if this current python file is used as main, execute the function 'main'
if __name__ == "__main__":

  main(sys.argv[1:])



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv

B=argv[1]

df_list=[]
for i in range(1,7):
    b=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/sheath_data_ascii/'+B+'%d.txt' % i,sep='\s+',header=None,names=['theta_1','theta_2','f_1','f_2','B','ni','Ti','V'],skiprows=1)
    b['phi1']=np.where(b.theta_1<=180,b.theta_1,b.theta_1-360)
    b['phi2']=np.where(b.theta_2<=180,b.theta_2,b.theta_2-360)
    b['phi']=(b.phi1+b.phi2)/2
    b['f']=(b.f_1+b.f_2)/2

    temp=b.drop(['theta_1','theta_2','f_1','f_2','phi1','phi2'],axis=1)
    temp.columns=['B_T','ni_T','Ti_T','V_T','phi','f']
    temp.to_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/sheath_data_ascii/'+B+'_Sp_%d.txt'% i,float_format='%.4f',index=False)
    df_list.append(temp)
d=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa.txt')
d_2=d[d.phi!=0].copy()
d_2.phi=-d_2.phi
d=pd.concat([d,d_2])

df_list2=[]
for i in range(1,7):
    temp=d.copy()
    temp.ni=temp.ni*5*i
    temp.V=temp.V*400
    temp.Ti=temp.Ti*2.586
    temp2=pd.merge(df_list[i-1],temp,on=['phi','f'])
    temp2.to_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/comp_'+B+'_'+str(1200+1200*i).zfill(6),float_format='%.3f')
    df_list2.append(temp2)



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
from sympy.solvers import solve
from sympy import Symbol,nsolve
from sympy import sqrt,log,tanh
from scipy.interpolate import NearestNDInterpolator as NNDI
from scipy.interpolate import LinearNDInterpolator as LNDI
from scipy.interpolate import RegularGridInterpolator as RGI
from scipy.spatial.transform import Rotation as R
import glob
import datetime

#For LNDI:
#np.save('test',f)
#f2=np.load('test.npy')
#f2.item()(x)

# Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

R_0=15.02
lam=1.17
epsilon=6.55

#filepath

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
    ptsnew[:,4] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew

def bsdf2(N,V,Ma,B,xl,yl,zl):
    D=0.937*(0.846+0.042*B)
    f=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
    t=0
    sol0=a44
    sol=sol0
    while sol*sol0>0:
        t+=0.001
        x=xl*t
        y=yl*t
        z=zl*t
        sol=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
#        print(sol)
    dist=np.linalg.norm(np.array([xl*t,yl*t,zl*t]))
    return dist

def bsdf_Jelinek(Pd,xyz):
    a_14=(4*R_0/lam**2*Pd**(-1/epsilon))
    a_44=(-1/4*lam**2*a_14**2)
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    cos=np.cos(theta)
    sin=np.sin(theta)
    r=np.array((-a_14*cos+np.sqrt(a_14**2*cos**2-4*a_44*sin**2))/(2*sin**2))
    r0=-a_44/a_14
    r[np.isnan(r)]=r0[np.isnan(r)]
    return r


def mpdf(Pd,Bz,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    r0=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    R = r0*(2/(1+np.cos(theta)))**alpha
    return R



def OMNIread2(f):

    BErr='9999.99'          # err of B field
    VErr='99999.9'          # err of Velocity
    NErr='999.99'               # err of Number density

    time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            vx.append(float(w[7]))
            vy.append(float(w[8]))
            vz.append(float(w[9]))
            n.append(float(w[10]))
            Pd.append(float(w[11]))
            Ma.append(float(w[12]))
            Mm.append(float(w[13]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    vx=np.array(vx)
    vy=np.array(vy)
    vz=np.array(vz)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    Mm=np.array(Mm)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    V=np.sqrt(vx**2+vy**2+vz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw})
    return omni

def WINDread(f):

    BErr='9999.99'          # err of B field
    VErr='99999.9'          # err of Velocity
    NErr='999.99'               # err of Number density

    time,Bx,By,Bz,V,n,Pd,Ma=[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            V.append(float(w[7]))
            n.append(float(w[8]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    V=np.array(V)
    n=np.array(n)
    Pd=2e-6*n*V**2
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    Ma=20*B_sw/np.sqrt(n)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'n':n,'Pd':Pd,'Ma':Ma,'B':B_sw})
    return omni

def SSCread(f):
    dateparse=lambda x,y: pd.datetime.strptime(x+' '+y,'%y/%m/%d %H:%M:%S')
    sat=pd.read_csv(f,header=None,sep='\s+',names=['d','t','x','y','z'],parse_dates={'datetime':[0,1]},date_parser=dateparse)
    sat.datetime=pd.to_datetime(sat.datetime)
    return sat

def main(xyz,f_sw,fout,model='jel'):
    if type(xyz)==str:
        xyz=SSCread(xyz)
        if (xyz.x<0).any():
            print('Warning: there is at least one time at which satellite was on nightside, which isn\'t supported by current version of model.')
            #print(xyz.x.max())

    omni=OMNIread2(f_sw)
    if len(omni)==1:
        omni=pd.concat([omni]*np.shape(xyz)[0])

    if type(xyz)!=np.ndarray:
        test=xyz.merge(omni,how='outer')[['datetime','x','y','z']].sort_values(by=['datetime']).set_index('datetime').interpolate('index')
        omni=omni.set_index('datetime').join(test,how='left')
        xyz_np=omni[['x','y','z']].to_numpy()
    else:
        xyz_np=xyz

    r=R.from_euler('z',-4,degrees=True)
    xyz_np=(r.as_matrix()@xyz_np.T).T

    print('Finding MP nodes')
    mpd=np.array(mpdf(omni.Pd,omni.Bz,xyz_np))
#    bsd=np.array([bsdf(omni.n[i],v_sw[i],Ma[i],B_sw[i],xyz[i])for i in range(np.shape(xyz)[0])])

    print('Finding BS nodes')
    if model=='jer':
        bsdfv=np.vectorize(bsdf2)
        bsd=np.array(bsdfv(omni.n,omni.V,omni.Ma,omni.B,xyz_np[:,0],xyz_np[:,1],xyz_np[:,2]))
    else:
        bsd=bsdf_Jelinek(omni.Pd,xyz_np)
    print(np.shape(mpd),np.shape(bsd))

    pts=appendSpherical_np(xyz_np)
    print('theta range',np.nanmin(pts[:,4])*180/np.pi,np.nanmax(pts[:,4])*180/np.pi)
    print('phi range',np.nanmin(pts[:,5]*180/np.pi),np.nanmax(pts[:,5])*180/np.pi)

    f_msh=(pts[:,3]-mpd)/(bsd-mpd)
#    print(type(pts),type(f_msh))
    pts=np.hstack((pts,f_msh.reshape(pts.shape[0],1)))
    #print(pts)

    pts_df=pd.DataFrame(pts, columns=['x','y','z','r','theta','phi','f'])
    input=pts_df[['theta','phi','f']].to_numpy()


    out=pts_df
    out.to_csv(fout,float_format='%.3f',index=False)

#    return pts_df
    return


if __name__=="__main__":
    f_sw="SW_cond_test.txt"
    test=open(f_sw,"w")
    test.write("2003 124 12 30    1.01   -1.65   -0.78  -414.8   -21.8     0.7   7.54  2.60  24.3  7.7\n")
    #Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
    test.close()

    f_sw='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/omni.lst'
    xyz=np.array([[12,2,3],[10,2,3]])
    #Orbit data from the SSWweb can be used as xyz. (filepath)
    #Data would be interpolated based on OMNI time.
    xyz='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/cl_orbit.txt'
    fout='Msh_test_out.txt'
    fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/Msh_orbit_test.txt'
    main(xyz,f_sw,fout)



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
from scipy import stats

n=6
run=sys.argv[1]
lf=int(sys.argv[2])
if run=='Msh_Nstep':external='easystore'
else:external='Elements'

df_list2=[]
for i in range(1,7):
    df_list=[]
    for phi in np.arange(-90+7.5,90,7.5):
        a=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/comp_phi_0.1_%.1f_' % phi+str(1200+1200*i).zfill(6))
        if lf==1: a=a[(a.f>=.3) & (a.f<=.7)]
        df_list.append(a)
    a2=pd.concat(df_list)
    df_list2.append(a2)
a3=pd.concat(df_list2)

list=['B','ni','Ti','V']
unit=['[nT]','[cm$^{-3}$]','[eV]','[km/s]']

label_l=np.arange(5,35,5)
for j in range(4):
    reg=[]
    fig,ax=plt.subplots(2,3,figsize=(10,10))
    ax_t=ax.flatten()
    for i in range(6):
        mask=~np.isnan(df_list2[i][list[j]+'_T'])&~np.isnan(df_list2[i][list[j]])
        reg.append(stats.linregress(df_list2[i][list[j]+'_T'][mask],df_list2[i][list[j]][mask]))

        ax_t[i].scatter(df_list2[i][list[j]+'_T'],df_list2[i][list[j]])
        ax_t[i].plot(df_list2[i][list[j]+'_T'][mask],reg[i].slope*df_list2[i][list[j]+'_T'][mask]+reg[i].intercept,label='y={:.2f}x+{:.2f}'.format(reg[i].slope,reg[i].intercept))
#        ax_t[i].annotate('y={:.2f}x+{:.2f}'.format(reg[i].slope,reg[i].intercept),xy=(.05,.95),xycoords='axes fraction')
        ax_t[i].plot(df_list2[i][list[j]+'_T'][mask],df_list2[i][list[j]+'_T'][mask],label='y=x')
        ax_t[i].grid()
        ax_t[i].legend()
        ax_t[i].set_xlabel(list[j]+'_THEMIS '+unit[j])
        ax_t[i].set_ylabel(list[j]+'_MHD '+unit[j])
        ax_t[i].set_title(str(label_l[i])+'cm$^{-3}$')
#    handles, labels = ax_t[i].get_legend_handles_labels()
#    fig.legend(handles, labels, loc='upper right',bbox_to_anchor=(0.9,0.99))
    #plt.suptitle('B_z=-4nT, 0.3$\leq$f$\leq$0.7, Maximum current')
    if run=='Msh_Nstep': plt.suptitle('B_z=4nT '+list[j]+' Comparison')
    else: plt.suptitle('B_z=-4nT '+list[j]+' Comparison')
    fig.tight_layout(rect=[0, 0.03, 1, 0.95])
    plt.savefig('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/plots/comp/Scatter_0.1'+list[j]+'.png')



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import pickle as pl
import io
from sys import argv
from read_mhd                import *
from pkg_mhdplot             import mhd_vpkg
from openggcm_info           import runinfo,mhdtime,mhdt2tstr
from matplotlib.colors       import LogNorm
from matplotlib.ticker       import MultipleLocator, FuncFormatter
from mpl_toolkits.axes_grid1 import make_axes_locatable

if argv[1]=='cs':
    run='Msh_Nstep_cs'
    external='Elements'
else:
    run='Msh_Nstep'
    external='easystore'

x=np.linspace(0,20)

for i in range(1,7):
    a=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_data_0.1.'+str(1200+1200*i).zfill(6),header=None,sep='\s+',names=['the','phi','m','rad','x','y','z','bx1','by1','bz1','vx1','vy1','vz1','rr1','pp1','ex1','ey1','ez1','xjx1','xjy1','xjz1','resis1']).drop_duplicates()
    a2=a[a.the==0]

    b=pd.read_csv('/Volumes/Elements/openggcm_run/'+run+'_003/modeling/Msh_data_0.1.'+str(1200+1200*i).zfill(6),header=None,sep='\s+',names=['the','phi','m','rad','x','y','z','bx1','by1','bz1','vx1','vy1','vz1','rr1','pp1','ex1','ey1','ez1','xjx1','xjy1','xjz1','resis1']).drop_duplicates()
    b2=b[b.the==0]

    for phi in np.arange(-90+7.5,90,7.5):
        y=x*np.tan(phi*np.pi/180)


        a_list,b_list=[],[]
        for j in range(1,10):
            a_t=a2[(a2.phi<phi+7.5) & (a2.phi>phi-7.5)]
            a_t2=a_t[(a_t.m>j-.1) & (a_t.m<j+.1)]
            a_t3=a_t2.mean().to_frame().T
            a_t3.phi=phi
            a_t3['f']=j/10
            a_list.append(a_t3)
            b_t=b2[(b2.phi<phi+7.5) & (b2.phi>phi-7.5)]
            b_t2=b_t[(b_t.m>j-.1) & (b_t.m<j+.1)]
            b_t3=b_t2.mean().to_frame().T
            b_t3.phi=phi
            b_t3['f']=j/10
            b_list.append(b_t3)

        a3=pd.concat(a_list,axis=0).set_index('f')
        a3['B']=np.sqrt(a3.bx1**2+a3.by1**2+a3.bz1**2)
        a3['Ti']=72429.0*a3.pp1/a3.rr1/11600.
        a3['V']=np.sqrt(a3.vx1**2+a3.vy1**2+a3.vz1**2)
        a3['ni']=a3.rr1
        a6=a3[['B','ni','Ti','V']]

        b3=pd.concat(b_list,axis=0).set_index('f')
        b3['B4']=np.sqrt(b3.bx1**2+b3.by1**2+b3.bz1**2)
        b3['Ti4']=72429.0*b3.pp1/b3.rr1/11600.
        b3['V4']=np.sqrt(b3.vx1**2+b3.vy1**2+b3.vz1**2)
        b3['ni4']=b3.rr1
        b6=b3[['B4','ni4','Ti4','V4']]


        comp=pd.concat([a6,b6],axis=1)
        comp2=comp[['B4','B','ni4','ni','Ti4','Ti','V4','V']]
        comp2.to_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/comp_phi_0.1_temp_%.1f_' % phi+str(1200+1200*i).zfill(6),float_format='%.3f')



import os, locale
import numpy as np
import matplotlib.pyplot as plt
#from mpl_toolkits.basemap import cm
from math import log10


def rate_vpkg(val):
  "set colormap, title, and range of variables for mhd plot"

####### xray values
  if val=='xray':
    cmap=plt.cm.jet
    name='Xray Intensity [$keV{}cm^{-2}s^{-1}sr^{-1}$]'
    name='[$keV{}cm^{-2}s^{-1}sr^{-1}$]'
    vran=[0,15]
    nlev=51
    tick=5
    ufac=1e-3
    norm=0

  elif val=='xraycut':
    cmap=plt.cm.jet
    name='Xray Rate [keVcm$^{-3}$s$^{-1}$]'
    vran=[0.,0.1]
    nlev=51
    tick=0.01
    ufac=1e-3
    norm=0

#   name='[LOG $eV{}cm^{-2}s^{-1}sr^{-1}$]'
#   vran=[100,100000]
#   tick=10.
#   ufac=1.
#   norm=1

  elif val=='Qxray' or val=='Qxray_nhmax' or val=='Qxray_nhmin':
    cmap=plt.cm.jet
#   cmap=plt.cm.gnuplot
    name='Q [10$^{20}$ cm$^{-4}$s$^{-1}$]'
    vran=[0.0,4.0]
    nlev=51
    tick=1.0
    ufac=1e-20
    norm=0

  elif val=='sQxray':	# strong Qxray
    cmap=plt.cm.jet
#   cmap=plt.cm.gnuplot
    name='Q [10$^{20}$ cm$^{-4}$s$^{-1}$]'
    vran=[0.,20.]
    nlev=51
    tick=4.
    ufac=1e-20
    norm=0

#   name='LOG Q [cm$^{-4}$s$^{-1}$]'
#   vran=[0.5e20,1e20]
#   tick=0.1e20
#   ufac=1.
#   norm=1

  elif val=='dxray':
    cmap=plt.cm.bwr
    name='$\Delta$ Q [10$^{20}$ cm$^{-4}$s$^{-1}$]'
    vran=[-0.5,0.5]
    nlev=51
    tick=0.1
    ufac=1e-20
    norm=0

  elif val=='ena' or val=='ena_nhmax' or val=='ena_nhmin':
    cmap=plt.cm.jet
    name='[$10^3 cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
#   name='ENA Flux [$10^3 cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
    vran=[0.,50.]
    nlev=51
    tick=10.
    ufac=1.e-3
    norm=0

#   name='[LOG $cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
#   vran=[100,100000]
#   ufac=1.
#   tick=10
#   norm=1

#   name='ENA Flux [$cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
#   vran=[0.,10]
#   tick=2.
#   ufac=1.

#   name='ENA Flux [$cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
#   vran=[1e3,1e5]
#   tick=10.
#   ufac=1.
#   norm=1

  elif val=='ena_5keV':
    cmap=plt.cm.jet
    name='[LOG $cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
    name='[$cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
#   name='ENA Flux [$10^3 cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
    vran=[0.1,100000]
    nlev=51
    tick=10
    ufac=1. #.e-3
    norm=1.

  elif val=='ena_10keV':
    cmap=plt.cm.jet
    name='[LOG $cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
    name='[$cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
#   name='ENA Flux [$10^3 cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
    vran=[0.1,10000]
    nlev=51
    tick=10
    ufac=1. #.e-3
    norm=1.

  elif val=='dena':
    cmap=plt.cm.bwr
    name='$\Delta$ ENA Flux [$10^3 cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
    name='[$10^3 cm^{-2}s^{-1}sr^{-1}keV^{-1}$]'
    vran=[-5.,5.]
    nlev=51
    tick=1.
    ufac=1e-3
    norm=0


  elif val=='Rcount':
    cmap=plt.cm.jet
#   cmap=plt.cm.hot
    name='Xray count [$counts min^{-1} deg^{-2}$]'
    name='[counts/pix]'
    vran=[0.,100.]
    nlev=51
    tick=20.
    ufac=1.
    norm=0

  elif val=='Bin_count':
    cmap=plt.cm.jet
    name='[counts/pix]'
    vran=[0.,200.]
    nlev=51
    tick=40.
    ufac=1.
    norm=0

#   vran=[1e1,1e3]
#   tick=1e1
#   norm=1


  elif val=='Kcount':
    cmap=plt.cm.hot
    cmap=plt.cm.jet
    name='ROSAT Xray count [$counts min^{-1} deg^{-2}$]'
    name='[counts/$0.25^{2{}\circ}$pix/$2$ min]'
    vran=[0.,150.]
    nlev=51
    tick=30.
    ufac=1.
    norm=0


  elif val=="prec_e_fe_1":
    cmap=plt.cm.jet                             # name of colormap
    name='Energy Flux [mW/m$^2$]'               # name of colorbar title
    name='[mW/m$^2$]'               # name of colorbar title
    vran=[0.1,300]                              # default range of value
    nlev=51                                     # number of contour level
    tick=10                                     # interval of major ticks for colorbar
    ufac=1.e3                                   # unit factor for a val in a desired unit
    norm=1                                      # 0 for linear-scale, 1 for log-scale



  elif val=='nsw':
    cmap=plt.cm.jet
    name='N [cm$^{-3}$]'
    name='[$cm^{-3}$]'
    vran=[0.,40.]
    nlev=51
    tick=10.
    ufac=1.
    norm=0

  elif val=='nn':
    cmap=plt.cm.jet
    cmap=plt.cm.gnuplot
    name='Nn [cm$^{-3}$]'
    vran=[0.,50.]
    nlev=51
    tick=10
    ufac=1.
    norm=0

  elif val=='vsw':
    cmap=plt.cm.jet
    name='Vsw [km/s]'
    vran=[0.,400.]
    nlev=51
    tick=50
    ufac=1.
    norm=0

  elif val=='vx':
    cmap=plt.cm.bwr
    name='[km/s]'
    vran=[-400.,400.]
    nlev=51
    tick=200
    ufac=1.
    norm=0


  elif val=='vth':
    cmap=plt.cm.jet
    name='Vth [km/s]'
    vran=[0.,800.]
    nlev=51
    tick=100
    ufac=1.
    norm=0

  elif val=='vav':
    cmap=plt.cm.jet
    cmap=plt.cm.gnuplot
    name='<G> [km/s]'
    vran=[100.,500.]
    nlev=51
    tick=100
    ufac=1.
    norm=0

  elif val=='tev':
    cmap=plt.cm.jet
    name='T [eV]'
    vran=[0.,400.]
    nlev=51
    tick=50
    ufac=1.
    norm=0

  elif val=='bt':
    cmap=plt.cm.jet
    cmap=plt.cm.gnuplot
    name='|B| [nT]'
    vran=[0.,100.]
    nlev=51
    tick=20
    ufac=1.
    norm=0

  elif val=='bx':
    cmap=plt.cm.bwr
    name='Bx [nT]'
    vran=[-100,100.]
    nlev=51
    tick=20
    ufac=1.
    norm=0

  elif val=='rr':
#   cmap=plt.cm.Greys
    cmap=plt.cm.jet
    name='Np [cm$^{-3}$]'
    vran=[0.,40.]
    nlev=51
    tick=10.
    ufac=1.
    norm=0

  elif val=="jt":
    cmap=plt.cm.jet
    cmap=plt.cm.gnuplot
    name='|J| [nA/m$^2$]'
    vran=[0.0,10.0]
    nlev=51
    tick=2.
    ufac=1.
    norm=0

  else  :		# defalut values
    cmap=plt.cm.jet
    name=val+' [?]'
    vran=[0,100.0]
    nlev=51
    tick=20.
    ufac=1.
    norm=0


  if norm==0:   # for linear-scale contour
     dlev = (vran[1]-vran[0])/(nlev-1)		 	# interval of contour level
     clev = np.arange(vran[0],vran[1]+dlev,dlev)	# contour levels
     tlev = np.arange(vran[0],vran[1]+tick,tick) 	# tick levels for the colorbar

  if norm==1: 	# for log-scale contour
     t1=log10(vran[0])
     t2=log10(vran[1])
     dt=log10(tick)
     print("norm vran: v1,v2,dv=",t1,t2,dt)

     dlev=(t2-t1)/(nlev-1)
     clev=[ 10**i for i in np.arange(t1,t2+dlev,dlev)]
     tlev=[ 10**i for i in np.arange(t1,t2+dt,dt) ]


  pkg_plot=[name,cmap,ufac,clev,tlev,norm]

  return pkg_plot



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

for i in range(1,7):
    a=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/modeling/Msh_data.'+str(1200+1200*i).zfill(6),header=None,sep='\s+',names=['the','phi','m','rad','x','y','z','bx1','by1','bz1','vx1','vy1','vz1','rr1','pp1','ex1','ey1','ez1','xjx1','xjy1','xjz1','resis1']).drop_duplicates()
    a2=a[a.the==0]
    b=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/sheath_data_ascii/cn%d.txt' % i,sep='\s+',header=None,names=['theta_1','theta_2','f_1','f_2','B','ni','Ti','V'],skiprows=1)
    b['theta']=(b.theta_1+b.theta_2)/2
    b['f']=(b.f_1+b.f_2)/2
    b['phi']=np.where(b.theta<=180,b.theta,b.theta-360)
    b2=b[(-90<=b.phi) & (b.phi<=90)]

    for j in range(1,10):
        f=j/10
        b3=b2[(b2.f<f+.01) & (b2.f>f-.01)]
        b4=b3[['phi','B','ni','Ti','V']].set_index('phi')
        b4.columns=['B_T','ni_T','Ti_T','V_T']

        a3=a2[a2.m==10*j].set_index('phi')
        a4=a3.reindex(a3.index.union(b3.phi))
        a5=a4.interpolate('index').reindex(index=b3.phi)
        a5['B']=np.sqrt(a5.bx1**2+a5.by1**2+a5.bz1**2)
        a5['Ti']=72429.0*a5.pp1/a5.rr1/11600.
        a5['V']=np.sqrt(a5.vx1**2+a5.vy1**2+a5.vz1**2)
        a5['ni']=a5.rr1
        a6=a5[['B','ni','Ti','V']]

        comp=pd.concat([a6,b4],axis=1)
        comp2=comp[['B_T','B','ni_T','ni','Ti_T','Ti','V_T','V']]
        comp2.to_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/modeling/comp_%.1f.' % j+str(1200+1200*i).zfill(6),float_format='%.3f')

        fig, axs=plt.subplots(4,figsize=(6.4,8))
        axs[0].plot(comp2.index,comp2.B_T,'o-')
        axs[0].plot(comp2.index,comp2.B,'o-')
        axs[1].plot(comp2.index,comp2.ni_T,'o-')
        axs[1].plot(comp2.index,comp2.ni,'o-')
        axs[2].plot(comp2.index,comp2.Ti_T,'o-')
        axs[2].plot(comp2.index,comp2.Ti,'o-')
        axs[3].plot(comp2.index,comp2.V_T,'o-')
        axs[3].plot(comp2.index,comp2.V,'o-')
        axs[3].set_xlabel('$\phi(^\circ)$ ')
        axs[0].set_ylabel('B (nT)')
        axs[1].set_ylabel('n (cm$^{-3}$)')
        axs[2].set_ylabel('T (eV)')
        axs[3].set_ylabel('V (km/s)')
        axs[0].grid()
        axs[1].grid()
        axs[2].grid()
        axs[3].grid()
        fig.suptitle('f=%.1f' % f)
        fig.savefig('/Volumes/easystore/openggcm_run/Msh_Nstep/modeling/plots/comp/comp_%d_m_f%.1f.png' % (20*i+20,f))
        plt.close()



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
from sympy.solvers import solve
from sympy import Symbol,nsolve
from sympy import sqrt,log,tanh
from scipy.spatial.transform import Rotation as R
from scipy.interpolate import RegularGridInterpolator as RGI
import glob
import datetime
from scipy.interpolate import interp2d
from astropy.io import fits
import pkg_resources

# Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

R_0=15.02
lam=1.17
epsilon=6.55

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
    ptsnew[:,4] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew

def bsdf2(N,V,Ma,B,xl,yl,zl):
    D=0.937*(0.846+0.042*B)
    f=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
    t=0
    sol0=a44
    sol=sol0
    while sol*sol0>0:
        t+=0.001
        x=xl*t
        y=yl*t
        z=zl*t
        sol=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
#        print(sol)
    dist=np.linalg.norm(np.array([xl*t,yl*t,zl*t]))
    return dist

def bsdf_Jelinek(Pd,xyz):

    a_14=(4*R_0/lam**2*Pd**(-1/epsilon))
    a_44=(-1/4*lam**2*a_14**2)
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    cos=np.cos(theta)
    sin=np.sin(theta)
    r=np.array((-a_14*cos+np.sqrt(a_14**2*cos**2-4*a_44*sin**2))/(2*sin**2))
    r0=-a_44/a_14
    r[np.isnan(r)]=r0[np.isnan(r)]
    return r

def mpdf(Pd,Bz,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    r0=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    R = r0*(2/(1+np.cos(theta)))**alpha
    return R

def OMNIread2(f):

    BErr='nan'          # err of B field
    VErr='nan'          # err of Velocity
    NErr='nan'               # err of Number density

    time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            vx.append(float(w[7]))
            vy.append(float(w[8]))
            vz.append(float(w[9]))
            n.append(float(w[10]))
            Pd.append(float(w[11]))
            Ma.append(float(w[12]))
            Mm.append(float(w[13]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    vx=np.array(vx)
    vy=np.array(vy)
    vz=np.array(vz)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    Mm=np.array(Mm)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    V=np.sqrt(vx**2+vy**2+vz**2)
    Tev=72429*Pd/n/11600
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw,'Tev':Tev})
    return omni

def WINDread(f):

    BErr='9999.99'			# err of B field
    VErr='99999.9'			# err of Velocity
    NErr='999.99'				# err of Number density

    time,Bx,By,Bz,V,n,Pd,Ma=[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            V.append(float(w[7]))
            n.append(float(w[8]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    V=np.array(V)
    n=np.array(n)
    Pd=2e-6*n*V**2
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    Ma=20*B_sw/np.sqrt(n)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'n':n,'Pd':Pd,'Ma':Ma,'B':B_sw})
    return omni

def SSCread(f):
    dateparse = lambda x, y: pd.to_datetime(x + ' ' + y, format='%y/%m/%d %H:%M:%S')
    sat = pd.read_csv(f, header=None, sep='\s+', names=['d','t','x','y','z'])
    sat['datetime'] = pd.to_datetime(sat['d'] + ' ' + sat['t'], format='%y/%m/%d %H:%M:%S')
    sat = sat.drop(['d', 't'], axis=1)
    return sat

def main(xyz,f_sw,fout,mpdo=0,bsdo=0,model='jel'):
    mpdo=float(mpdo)
    bsdo=float(bsdo)

    if type(xyz)==str:
        xyz=SSCread(xyz)
        if (xyz.x<0).any():
            print('Warning: there is at least one time at which satellite was on nightside, which isn\'t supported by current version of model.')

    omni=OMNIread2(f_sw)
    if len(omni)==1:
        omni=pd.concat([omni]*np.shape(xyz)[0])

    print('SW temp mean',omni.Tev.mean())

    if type(xyz)!=np.ndarray:
        test=xyz.merge(omni,how='outer')[['datetime','x','y','z']].sort_values(by=['datetime']).set_index('datetime').interpolate('index')
        omni=omni.set_index('datetime').join(test,how='left')
        xyz_np=omni[['x','y','z']].to_numpy()
    else:
        xyz_np=xyz

#    print(len(xyz_np))
#    print(len(omni))

    r=R.from_euler('z',-4,degrees=True)
    xyz_np=(r.as_matrix()@xyz_np.T).T

    print('Finding MP nodes')
    mpd=np.array(mpdf(omni.Pd,omni.Bz,xyz_np))
    mpd+=mpdo

    print('Finding BS nodes')
    if model!='jel':
        bsdfv=np.vectorize(bsdf2)
        bsd=np.array(bsdfv(omni.n,omni.V,omni.Ma,omni.B,xyz_np[:,0],xyz_np[:,1],xyz_np[:,2]))
    else:
        bsd=bsdf_Jelinek(omni.Pd,xyz_np)
    bsd+=bsdo

    pts=appendSpherical_np(xyz_np)
    print('theta range',np.nanmin(pts[:,4])*180/np.pi,np.nanmax(pts[:,4])*180/np.pi)
    print('phi range',np.nanmin(pts[:,5]*180/np.pi),np.nanmax(pts[:,5])*180/np.pi)

#    print(np.shape(pts[:,3]),mpd,bsd)
    f_msh=(pts[:,3]-mpd)/(bsd-mpd)
    pts=np.hstack((pts,f_msh.reshape(pts.shape[0],1)))

    pts_df=pd.DataFrame(pts, columns=['x','y','z','r','theta','phi','f'])
    pts_df['phi2']=np.absolute(pts_df.phi)
    input=pts_df[['phi2','f']].to_numpy()

    if (omni.n>35).any(): print('Warning: there is at least one time at which solar wind density goes above 35cm^-3, in which our model was not validated')
    if (omni.B>15).any(): print('Warning: there is at least one time at which IMF magnitude goes above 15cm^-3, in which our model shows discrepancy with THEMIS data')

    phi=np.arange(0,128)*np.pi/180
    af=np.arange(0,10)/10

    n1=np.loadtxt(pkg_resources.resource_filename('Mshpy',f'data/Spreiter/Msh_n'))
    T1=np.loadtxt(pkg_resources.resource_filename('Mshpy',f'data/Spreiter/Msh_T'))
    V1=np.loadtxt(pkg_resources.resource_filename('Mshpy',f'data/Spreiter/Msh_V'))

    nf=RGI((phi,af),n1,bounds_error=False)
    Tf=RGI((phi,af),T1,bounds_error=False)
    Vf=RGI((phi,af),V1,bounds_error=False)

    n=nf(input)*omni.n.to_numpy()
    T=Tf(input)*omni.Tev.to_numpy()
    V=Vf(input)*omni.V.to_numpy()

    out=pd.DataFrame({'time':omni.index,'f':pts_df.f,'n':n,'Tev':T,'V':V})
    out.to_csv(fout,float_format='%.3f',index=False)

    return n,T,V

if __name__=='__main__':
    f_sw='SW_cond_test.txt'
    test=open(f_sw,"w")
    test.write("2003 124 12 30    1.01   -1.65   -0.78  -414.8   -21.8     0.7   7.54  2.60  24.3  7.7\n")
    #Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
    test.close()

    xyz=np.array([[12,2,3],[10,2,3]])
    fout='Msh_Spreiter_test_out.txt'
    main(xyz,f_sw,fout)



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
import datetime
import glob
import BSaMP


def OMNIread(f):

    BErr='9999.99'          # err of B field
    VErr='99999.9'          # err of Velocity
    NErr='999.99'               # err of Number density

    time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            vx.append(float(w[7]))
            vy.append(float(w[8]))
            vz.append(float(w[9]))
            n.append(float(w[10]))
            Pd.append(float(w[11]))
            Ma.append(float(w[12]))
            Mm.append(float(w[13]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    vx=np.array(vx)
    vy=np.array(vy)
    vz=np.array(vz)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    Mm=np.array(Mm)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    V=np.sqrt(vx**2+vy**2+vz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw})
    return omni

def SSCread(f):
    dateparse=lambda x,y: pd.datetime.strptime(x+' '+y,'%y/%m/%d %H:%M:%S')
    sat=pd.read_csv(f,header=None,sep='\s+',names=['d','t','x','y','z'],parse_dates={'datetime':[0,1]},date_parser=dateparse)
    sat.datetime=pd.to_datetime(sat.datetime)
    return sat

f_sw='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001_Jel/omni.lst'
xyz='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001_Jel/orbit.txt'
sc='Cluster 4'
omni=OMNIread(f_sw)
xyz=SSCread(xyz)

fig,(ax9,ax10)=plt.subplots(1,2)

BSaMP.BSMPj(ax9,ax10,omni.Bz[0],omni.Pd[0],0)
BSaMP.BSMPj(ax9,ax10,omni.Bz.iloc[-1],omni.Pd.iloc[-1],1)
ax9.plot(xyz.x,xyz.y,label=sc)
ax9.plot(xyz.x[0],xyz.y[0],'o',markersize=3)
ax9.set_xticks(np.linspace(-20,20,5))
ax9.grid()
ax9.set_ylabel('$Y_{gse}[Re]$')
ax9.set_xlabel('$X_{gse}[Re]$')
#ax9.text(-0.1,1.1,'A',transform=ax9.transAxes, size=20,weight='bold')
#    leg9=ax9.legend(handlelength=0,loc='upper left',bbox_to_anchor=(0,.9),framealpha=0,fontsize=9)
#    for line,text in zip(leg9.get_lines(),leg9.get_texts()):
#        text.set_color(line.get_color())
ax9.set_aspect('equal')


ax10.plot(xyz.x,xyz.z,label=sc)
ax10.plot(xyz.x[0],xyz.z[0],'o',markersize=3)
ax10.set_xticks(np.linspace(-20,20,5))
ax10.grid()
ax10.set_xlabel('$X_{gse}[Re]$')
ax10.set_ylabel('$Z_{gse}[Re]$')
#    leg10=ax10.legend(handlelength=0,loc='upper left',bbox_to_anchor=(0,.9),framealpha=0,fontsize=9)
#    for line,text in zip(leg10.get_lines(),leg10.get_texts()):
#        text.set_color(line.get_color())
ax10.set_aspect('equal')
plt.tight_layout()

plt.savefig('2003May04_001_sc_orbit_Jel.png')



#!/usr/bin/python
#
# plot xray rate

import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")

import math
import locale
import numpy              	as np
import matplotlib.pyplot  	as plt
from   openggcm_info            import *
from   read_mhd                 import *
from   mpl_toolkits.axes_grid1  import make_axes_locatable
from   pkg_rates                import rate_vpkg
#from   scipy                    import interpolate


def create_xray(fout,fgrd,f3df,dipt,scinfo,xrinfo):
  "create xray file seen from a virtual spacecraft			\
   scinfo : s/c location (xsc,ysc,zsc) and look direction (xlk,ylk,zlk)	\
   xrinfo : s/c field of view (the1,the2,dthe,phi1,phi2,dphi)		"
#### check if fout exists. If so, choose replacing or useing the old fout
  if os.path.isfile(fout):
    print('============================================================================')
    test=input('Files exist! Press (o/O) to Overwrite and (u/U) to use old ones :')
    if   test=='o' or test=='O' :
       run=True
       print('rm '+fout)
       os.system('rm '+fout)
    elif test=='u' or test=='U' :
       run=False
    else :
       print("Error! Press O for Overwrite, U for Using old file. Please Run again")
       sys.exit(1)
  else :
    run=True
#### calculate Xray
  if run:
    xsc,ysc,zsc,xlk,ylk,zlk       = scinfo
    the1,the2,dthe,phi1,phi2,dphi = xrinfo
#   xsc,ysc,zsc,xlk,ylk,zlk       = [ locale.atof(i) for i in scinfo.split(':') ]
#   the1,the2,dthe,phi1,phi2,dphi = [ locale.atof(i) for i in xrinfo.split(':') ]
    EXE = "$HOME/svn/hkc-project/trunk/src/xray_ena/xraysc_sight"
#   EXE = "$HOME/svn/hkc-project/trunk/src/xray_ena/xraysc_clipMP"
    OP1 = " -xsc %7.2f -ysc %7.2f -zsc %7.2f"%(xsc,ysc,zsc)
    OP2 = " -xlk %7.2f -ylk %7.2f -zlk %7.2f"%(xlk,ylk,zlk)
    OP3 = " -the1 %7.2f -the2 %7.2f -dthe %7.2f -phi1 %7.2f -phi2 %7.2f -dphi %7.2f"\
          %(the1,the2,dthe,phi1,phi2,dphi)
    OP4 = " -grid "+fgrd+" -fin "+f3df+" -diptime "+dipt+" -fo "+fout
    cmd = EXE+OP1+OP2+OP3+OP4
    os.system(cmd)
  return



def find_MPBScontour(fmhd,pn,lp,rr,sc):
  " find MP/BS location from density contour lines.            	\
    fmhd  : need 2DMHD data on either xz or xy plane		\
    rr    : number density to select bs & mp                 	"

  xx,yy,vv,vstr = read_2DMHD(fmhd,pn)
  x,y           = np.meshgrid(xx,yy)
  cden          = plt.contourf(x,y,vv[16],levels=np.arange(0,51.,1.),extend='both')
  csbs          = plt.contour(xx,yy,vv[16],levels=[rr[0]],colors=['m'])
  vbs1          = csbs.collections[0].get_paths()[0]
  vbs2          = vbs1.vertices

  csmp          = plt.contour(xx,yy,vv[16],levels=[rr[1]],colors=['r'])
  if len(csmp.collections[0].get_paths())==1:
    vmp1        = csmp.collections[0].get_paths()[0]
  else:
    vmp1        = csmp.collections[0].get_paths()[1]
  vmp2          = vmp1.vertices

  if   pn=='xz':
    xbs  = vbs2[:,0]
    ybs  = [ lp for i in xbs]
    zbs  = vbs2[:,1]
    xmp  = vmp2[:,0]
    ymp  = [ lp for i in xmp]
    zmp  = vmp2[:,1]
    mp   = plt.plot(xmp,zmp,'w')
    bs   = plt.plot(xbs,zbs,'m')
  elif pn=='xy':
    xbs  = vbs2[:,0]
    ybs  = vbs2[:,1]
    zbs  = [ lp for i in xbs]
    xmp  = vmp2[:,0]
    ymp  = vmp2[:,1]
    zmp  = [ lp for i in xmp]
    mp   = plt.plot(xmp,ymp,'w')
    bs   = plt.plot(xbs,ybs,'m')

  info = runinfo(fmhd)
  fps  = 'test.'+info.run+'.contour.'+info.mhdt+'.pdf'
  print("SAVE ",fps)
  plt.savefig(fps)
  plt.close()

  tmp=[];pmp=[]
  tbs=[];pbs=[]
  for i in range(len(xmp)):
    x    = xmp[i]-sc[0]
    y    = ymp[i]-sc[1]
    z    = zmp[i]-sc[2]
    rr1  = (x**2+y**2+z**2)**0.5
    the1 = 90.-math.degrees(math.acos(z/rr1))
    phi1 = math.degrees(math.atan2(y,x)) #-90.
    if phi1<0: phi1=360.+phi1
    tmp.append(the1)
    pmp.append(phi1)

  for i in range(len(xbs)):
    x    = xbs[i]-sc[0]
    y    = ybs[i]-sc[1]
    z    = zbs[i]-sc[2]
    rr1  = (x**2+y**2+z**2)**0.5
    the1 = 90.-math.degrees(math.acos(z/rr1))
    phi1 = math.degrees(math.atan2(y,x)) #-90.
    if phi1<0: phi1=360.+phi1
    tbs.append(the1)
    pbs.append(phi1)

  bdxsc = [pmp,tmp,pbs,tbs]
  if pn=='xz': bdgse = [xmp,zmp,xbs,zbs]
  if pn=='xy': bdgse = [xmp,ymp,xbs,ybs]
  mpbs = [bdxsc,bdgse]

  return mpbs


def read_mpbs(fi,pn,sc):
  "read and calculate the locations of BS and Mpause in the s/c point of view"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  tmp=[];pmp=[]
  tbs=[];pbs=[]
  for s in l[1:-1]:
    w=s.split(None)
    xbs=locale.atof(w[2])-sc[0]
    ybs=locale.atof(w[3])-sc[1]
    zbs=locale.atof(w[4])-sc[2]
    xmp=locale.atof(w[6])-sc[0]
    ymp=locale.atof(w[7])-sc[1]
    zmp=locale.atof(w[8])-sc[2]
### calculate the locations in the satellite's view
    rr1  = (xmp**2+ymp**2+zmp**2)**0.5
    the1 = 90.-math.degrees(math.acos(zmp/rr1))
    phi1 = math.degrees(math.atan2(ymp,xmp))
    if phi1<0: phi1=360.+phi1
    rr2  = (xbs**2+ybs**2+zbs**2)**0.5
    the2 = 90.-math.degrees(math.acos(zbs/rr2))
    phi2 = math.degrees(math.atan2(ybs,xbs))
    if phi2<0: phi2=360.+phi2
    tmp.append(the1)
    pmp.append(phi1)
    tbs.append(the2)
    pbs.append(phi2)

  bdxsc = [pmp,tmp,pbs,tbs]
  if pn=='xz': bdgse = [xmp,zmp,xbs,zbs]
  if pn=='xy': bdgse = [xmp,ymp,xbs,ybs]
  mpbs = [bdxsc,bdgse]
  return mpbs


def read_bline(fi,sc):
  "read and calculate bline in the s/c point of view"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  nl  = locale.atoi(l[-2].split(None)[0])
  blt = [ [] for i in range(nl) ]
  blp = [ [] for i in range(nl) ]
  oc  = []
  for s in l[:-1]:
    w    = s.split(None)
    il   = locale.atoi(w[0])-1
    blx1 = locale.atof(w[4])
    bly1 = locale.atof(w[5])
    blz1 = locale.atof(w[6])
    blx  = blx1-sc[0]
    bly  = bly1-sc[1]
    blz  = blz1-sc[2]
    rr   = (blx**2+bly**2+blz**2)**0.5
    the  = 90.-math.degrees(math.acos(blz/rr))
    phi  = math.degrees(math.atan2(bly,blx))
    if phi<0: phi=360.+phi
#   if il==1105 :print nl,il,blx,bly,blz,rr,the,phi
    blt[il].append(the)
    blp[il].append(phi)
    oc.append(locale.atoi(w[4]))
  f.close()
  bl = [blp,blt,oc]
  return bl


def read_cosmic(fi,plot=False):
  "read cosmic background of X-ray observed by ROSAT satellite			\
   INPUT  : X-ray cosmic background in unit of 10^-6 ROSAT counts/s/arcmin^2  	\
            111 x 111 pixels, 12 arcmin per pixel (1 arcmin = 1/60 deg)		\
   OuTPUT : return X-ray background in unit of counts/s/deg^2,			\
            22 x 22 pixels, 1 deg per pixel   					"

  f=open(fi,'r');l=f.read().split('\n');f.close()
  nx=111
  ny=111
  xx= [ i*12/60. for i in range(nx) ]
  yy= [ i*12/60. for i in range(nx) ]
  vv= np.ndarray((nx,ny))
  i=0;j=0
  for s in l[:-1]:
    w=s.split(None)
    for k in range(len(w)):
       vv[i][j] = locale.atof(w[k])*3600*1e-6  # convert to counts/s/deg^2
       j        = j+1
       if j==111: i=i+1;j=0
# bf  = interpolate.interp2d(xx,yy,vv,kind='linear')
# phi = range(xlen)
# lat = range(ylen)

# rate= bf(phi,lat).T
  if plot :
    fig  = plt.figure(figsize=(8,8))
    ax1  = plt.subplot(111,aspect='equal')
    X,Y  = np.meshgrid(xx,yy)
    pc1  = plt.pcolor(X,Y,vv)
    ax1.set_xlim(xx[0],xx[-1])
    ax1.set_ylim(yy[0],yy[-1])
    ax1.set_title(fi)
    cax1 = make_axes_locatable(ax1)
    cax2 = cax1.append_axes("right",size="3%",pad="5%")
    cb   = fig.colorbar(pc1,cax=cax2)
    plt.savefig('test.cosmic_bg.ps')
  return xx,yy,vv


def read_ena(fi):
  "read ena rates"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  s    = l[0].split(None)
  nt   = locale.atoi(s[0])
  np   = locale.atoi(s[1])
  tt   = locale.atof(s[2])
  info = [ locale.atof(i) for i in s[3:] ] # xsc,ysc,zsc,xlk,ylk,zlk,ena
  lat  = []
  phi  = []
  rate1= ndarray((np,nt))
  rate2= ndarray((np,nt))
  rate3= ndarray((np,nt))
  rate1.fill(nan)
  rate2.fill(nan)
  rate3.fill(nan)
  for s in l[1:]:
    w =s.split(None)
    if len(w)>2:
      i1   = locale.atoi(w[0])-1
      j1   = locale.atoi(w[1])-1
      rad1 = locale.atof(w[2])
      phi1 = locale.atof(w[3])
      lat1 = locale.atof(w[4])
      val1 = locale.atof(w[5])
      val2 = locale.atof(w[6])
      val3 = locale.atof(w[7])
      if i1==0       : lat.append(lat1)
      if j1==0       : phi.append(phi1)
      if val1!=9999.0 : rate1[j1][i1]=val1
      if val2!=9999.0 : rate2[j1][i1]=val2
      if val3!=9999.0 : rate3[j1][i1]=val3
  rate = [rate1,rate2,rate3]
  print(len(lat),len(phi))
  return phi,lat,rate,info


def read_xray(fi):
  "read xray rates in unit of eV cm-2 s-1 sr-1"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  s    = l[0].split(None)
  nt   = locale.atoi(s[0])
  np_   = locale.atoi(s[1])
  sc   = [ locale.atof(i) for i in s[2:] ]
  lat  = []
  phi  = []
  ival = False
  rate1= ndarray((np_,nt))
  rate2= ndarray((np_,nt))
  rate3= ndarray((np_,nt))
  rate1.fill(0)
  rate2.fill(0)
  rate3.fill(0)
# rate1.fill(nan)
# rate2.fill(nan)
# rate3.fill(nan)
  for s in l[1:]:
    w =s.split(None)
    if len(w)>2:
      i1   = locale.atoi(w[0])-1
      j1   = locale.atoi(w[1])-1
      rad1 = locale.atof(w[2])
      phi1 = locale.atof(w[3])
      lat1 = locale.atof(w[4])
      val1 = locale.atof(w[5])
      if len(w)==8:
        ival = True
        val2 = locale.atof(w[6])
        val3 = locale.atof(w[7])
        if val2!=9999.0 : rate2[j1][i1]=val2
        if val3!=9999.0 : rate3[j1][i1]=val3
      if i1==0       : lat.append(lat1)
      if j1==0       : phi.append(phi1)
      if val1!=9999.0: rate1[j1][i1]=val1

  if ival: rate=[rate1,rate2,rate3]
  else   : rate=rate1
  lat=np.array(lat)
  return phi,lat,rate,sc

def read_xraycut(fi):
  "read xray rates in unit of eV cm-2 s-1 sr-1"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  s    = l[0].split(None)
  nt   = locale.atoi(s[0])
  np_   = locale.atoi(s[1])
  sc   = [ locale.atof(i) for i in s[2:] ]
  lat  = []
  phi  = []
  ival = False
  rate1= ndarray((np_,nt))
  rate2= ndarray((np_,nt))
  rate3= ndarray((np_,nt))
  rate1.fill(0)
  rate2.fill(0)
  rate3.fill(0)
# rate1.fill(nan)
# rate2.fill(nan)
# rate3.fill(nan)
  for s in l[1:]:
    w =s.split(None)
    if len(w)>2:
      i1   = locale.atoi(w[0])-1
      j1   = locale.atoi(w[1])-1
      phi1 = locale.atof(w[2])
      lat1 = locale.atof(w[3])
      val1 = locale.atof(w[4])
      if i1==0       : lat.append(lat1)
      if j1==0       : phi.append(phi1)
      if val1!=9999.0: rate1[j1][i1]=val1

  if ival: rate=[rate1,rate2,rate3]
  else   : rate=rate1
  lat=np.array(lat)
  return phi,lat,rate

def read_LOS(fi):
  "read 1D data of LOS xray rates"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  s    = l[0].split(None)
  phi  = locale.atof(s[2])
  the  = locale.atof(s[3])
  lat  = 90.-the
  sc   = [ locale.atof(i) for i in s[4:] ]

  rad=[];rate1=[];rate2=[];rate3=[]
  x=[];y=[];z=[];bx=[];by=[];bz=[];vx=[];vy=[];vz=[];rr=[];pp=[]
  ex=[];ey=[];ez=[];xjx=[];xjy=[];xjz=[];tk=[];vth=[];vsw=[];vel=[]
  rrn1=[];rrn2=[];rrn3=[];xray1=[];xray2=[];xray3=[]

  for s in l[1:]:
    w =s.split(None)
    if len(w)>2:
      rad1,phi1,lat1,rate11,rate12,rate13,x1,y1,z1,bx1,by1,bz1,			\
      vx1,vy1,vz1,rr1,pp1,ex1,ey1,ez1,xjx1,xjy1,xjz1,temp1,vth1,vsw1,vel1,	\
      rrn11,rrn12,rrn13,xray11,xray12,xray13 = [locale.atof(i) for i in w[3:] ]

      rad.append(rad1)
      x.append(x1)
      y.append(y1)
      z.append(z1)
      bx.append(bx1)
      by.append(by1)
      bz.append(bz1)
      vx.append(vx1)
      vy.append(vy1)
      vz.append(vz1)
      rr.append(rr1)
      pp.append(pp1)
      ex.append(ex1)
      ey.append(ey1)
      ez.append(ez1)
      xjx.append(xjx1)
      xjy.append(xjy1)
      xjz.append(xjz1)
      tk.append(temp1)
      vth.append(vth1)
      vsw.append(vsw1)
      vel.append(vel1)
      rrn1.append(rrn11)
      rrn2.append(rrn12)
      rrn3.append(rrn13)
      xray1.append(xray11)
      xray2.append(xray12)
      xray3.append(xray13)
      rate1.append(rate11)
      rate2.append(rate12)
      rate3.append(rate13)
  pos  =[sc,phi,lat,rad,x,y,z]
  vmhd =[bx,by,bz,vx,vy,vz,ex,ey,ez,xjx,xjy,xjz,rr,pp,tk]
  vxr1 =[rr,vsw,vth,vel,tk,rrn1,rrn2,rrn3]
  vxr2 =[xray1,xray2,xray3,rate1,rate2,rate3]
  return pos,vmhd,vxr1,vxr2


def read_xcounts(fi):
  "read xray crount rates in unit of eV cm-2 s-1 sr-1"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  s    = l[0].split(None)
  nt   = locale.atoi(s[0])
  np   = locale.atoi(s[1])
  sc   = [ locale.atof(i) for i in s[2:] ]
  lat  = []
  phi  = []
  brate = ndarray((np,nt))
  xrate = ndarray((np,nt))
  xblur = ndarray((np,nt))
  xbfin = ndarray((np,nt))
  xbsig = ndarray((np,nt))
  for s in l[1:]:
    w =s.split(None)
    if len(w)>2:
      i1   = locale.atoi(w[0])-1
      j1   = locale.atoi(w[1])-1
      phi1 = locale.atof(w[2])
      lat1 = locale.atof(w[3])
      val1 = locale.atof(w[4])
      val2 = locale.atof(w[5])
      val3 = locale.atof(w[6])
      val4 = locale.atof(w[7])
      val5 = locale.atof(w[8])
      if i1==0       : lat.append(lat1)
      if j1==0       : phi.append(phi1)
      bprate[j1][i1] = val1
      xrate[j1][i1]  = val2
      xbblur[j1][i1] = val3
      xbfin[j1][i1]  = val4
      xbsig[j1][i1]  = val5
  rates = [brate,xrate,xblur,xbfin,xbsig]
  return phi,lat,rates,sc


def read_kipcounts(fi):
  "read Kip's xray crount rates in counts/pix for Texp"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  s    = l[0].split(None)
  nthe = locale.atoi(s[0])
  nphi = locale.atoi(s[1])
  sc   = [ locale.atof(i) for i in s[2:] ]	# xsc,ysc,zsc,xlk,ylk,zlk,Rpix,Texp
  lat  = []
  phi  = []
  xrexp  = ndarray((nthe,nphi))
  xcount = ndarray((nthe,nphi))
  xbrate = ndarray((nthe,nphi))
  xbblur = ndarray((nthe,nphi))
  xbinst = ndarray((nthe,nphi))
  xbfin  = ndarray((nthe,nphi))
  bgblur = ndarray((nthe,nphi))
  xbsig  = ndarray((nthe,nphi))

  for s in l[1:]:
    w =s.split(None)
    if len(w)>2:
      i1   = locale.atoi(w[0])
      j1   = locale.atoi(w[1])
      phi1 = locale.atof(w[2])
      lat1 = locale.atof(w[3])
      val1 = locale.atof(w[4])
      val2 = locale.atof(w[5])
      val3 = locale.atof(w[6])
      val4 = locale.atof(w[7])
      val5 = locale.atof(w[8])
      val6 = locale.atof(w[9])
      val7 = locale.atof(w[10])
      val8 = locale.atof(w[11])
      if i1==0       : phi.append(phi1)
      if j1==0       : lat.append(lat1)
      xrexp [i1][j1] = val1
      xcount[i1][j1] = val2
      xbrate[i1][j1] = val3
      xbblur[i1][j1] = val4
      xbinst[i1][j1] = val5
      xbfin [i1][j1] = val6
      bgblur[i1][j1] = val7
      xbsig [i1][j1] = val8

  print('max & min xrate  in Q:',np.nanmax(xrexp),np.nanmin(xrexp))
  print('max & min xcount in counts/pix/Texp:',np.nanmax(xcount),np.nanmin(xcount))
  print('max & min xbrate in counts/pix/Texp:',np.nanmax(xbrate),np.nanmin(xbrate))
  print('max & min xbblur in counts/pix/Texp:',np.nanmax(xbblur),np.nanmin(xbblur))
  print('max & min xbinst in counts/pix/Texp:',np.nanmax(xbinst),np.nanmin(xbinst))
  print('max & min xbfin  in counts/pix/Texp:',np.nanmax(xbfin),np.nanmin(xbfin))
  print('max & min xbsig  in counts/pix/Texp:',np.nanmax(xbsig),np.nanmin(xbsig))
  print('xcount mean,std:',np.mean(xcount),np.std(xcount))
  print('bgblur mean,std:',np.mean(bgblur),np.std(bgblur))
  print('xbrate mean,std:',np.mean(xbrate),np.std(xbrate))
  print('xbblur mean,std:',np.mean(xbblur),np.std(xbblur))
  print('xbinst mean,std:',np.mean(xbinst),np.std(xbinst))
  print('xbfin  mean,std:',np.mean(xbfin),np.std(xbfin))
  print('xbsig  mean,std:',np.mean(xbsig),np.std(xbsig))

  rates = [xrexp,xcount,xbrate,xbblur,xbinst,xbfin,bgblur,xbsig]
  return phi,lat,rates,sc

def read_xrbin(fi):
  "read rebinned xray crount rates in counts/pix for Texp"
  print("READ",fi)
  f=open(fi,'r');l=f.read().split('\n');f.close()
  s    = l[0].split(None)
  nthe = locale.atoi(s[0])
  nphi = locale.atoi(s[1])
  sc   = [ locale.atof(i) for i in s[2:] ]      # xsc,ysc,zsc,xlk,ylk,zlk,Rpix,Texp
  lat  = []
  phi  = []
  xrbin  = np.ndarray((nthe,nphi))

  for s in l[1:]:
    w =s.split(None)
    if len(w)>2:
      i1   = locale.atoi(w[0])
      j1   = locale.atoi(w[1])
      phi1 = locale.atof(w[2])
      lat1 = locale.atof(w[3])
      val1 = locale.atof(w[4])
      if i1==0       : phi.append(phi1)
      if j1==0       : lat.append(lat1)
      xrbin [i1][j1] = val1

  print('max & min xrbin in couns/pix/Texp:',np.nanmax(xrbin),np.nanmin(xrbin))
  print('xrbin mean,std:',np.mean(xrbin),np.std(xrbin))
  return phi,lat,xrbin,sc


def read_psf(fi,plot=False):
  "read Kip's point spred function.				\
   The pixcel size is selected to be 0.375 degree		"

  f=open(fi,'r');l=f.read().split('\n');f.close()
  nx  = len(l[0].split(None))
  ny  = len(l)-1
  x   = [0.375*i for i in np.arange(-30,31,1) ]
  y   = [0.375*i for i in np.arange(-30,31,1) ]
# x   = [0.1*i for i in np.arange(-30,31,1) ]
# y   = [0.1*i for i in np.arange(-30,31,1) ]
  psf = ndarray((nx,ny))
  psf.fill(0.)

  print(nx,ny)
  for i in range(ny):
    w=l[i].split(None)
    if len(w)>2:
      for j in range(len(w)):
        psf[i][j]= locale.atof(w[j])
#       if w[j]!=0 : psf[i][j]= np.log10(locale.atof(w[j]))
#       else       : psf[i][j]= -9999.

  if plot==True:
    vmax  = 0.
    vmin  = -10.
    tint  = 2.
    dlev  = (vmax-vmin)/50.
    clev  = np.arange(vmin,vmax+dlev,dlev)
    tlev  = np.arange(vmin,vmax+tint,tint)

    fig   = plt.figure()
    ax1   = fig.add_subplot(111,aspect='equal')
    xx,yy = np.meshgrid(x,y)
#   cs    = ax1.contourf(xx,yy,psf,levels=clev)
    cs    = ax1.scatter(xx,yy,c=np.log10(psf),vmin=vmin,vmax=vmax,cmap=plt.cm.jet,\
            edgecolors='none',marker='s')
    cax1  = make_axes_locatable(ax1)
    cax2  = cax1.append_axes("right",size="3%",pad="5%")
    cb    = fig.colorbar(cs,cax=cax2,ticks=tlev)
    ax1.set_xlabel("0.375$^{\circ}$ pixels")
    ax1.set_ylabel("0.375$^{\circ}$ pixels")
    ax1.set_title('SMILE Point Spread Function')
    plt.savefig('test_psf.pdf')

  return x,y,psf


def main(argv):

  fi       = 'rass_3a.txt'
# fi       = 'rass_3b.txt'
# xx,yy,vv = read_cosmic(fi,plot=True)
  fi       = read_psf('temp_psf_t0.txt',plot=True)
  x,y,zz   = read_psf(fi)

  return




if __name__=='__main__':
  main(sys.argv[1:])



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.interpolate import interp2d
from astropy.io import fits

cd=1/156
fits_image_filename_n='/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa_density.fits'
hdu_n=fits.open(fits_image_filename_n)
data_n=hdu_n[0].data
fits_image_filename_v='/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa_velocity.fits'
hdu_v=fits.open(fits_image_filename_v)
data_v=hdu_v[0].data
data_T=(data_v!=0)*(1+64/3*(1-data_v**2))
x=np.array([1-(i-133)*cd for i in range(512)])
y=np.array([(i-10)*cd for i in range(491)])
n=interp2d(x,y,data_n)
v=interp2d(x,y,data_v)
print(np.min(x),np.max(x))
print(np.min(y),np.max(y))

path='/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/BS2.txt'

def BS2(path_):
    a=open(path_,"w")
    n_r=0
    for phi in np.arange(0,181):
#        print(phi)
        r=0
        r_mp=0
        while (n_r==0) & (r<=3.6):
            n_r=n(r*np.cos(phi*np.pi/180),r*np.sin(phi*np.pi/180))
            r+=.01
        r_mp=r
        if n_r>0:
            while n_r>0:
                n_r=n(r*np.cos(phi*np.pi/180),r*np.sin(phi*np.pi/180))
                r+=.01
            a.write('%f %f %f\n'%(phi,r_mp,r))
    a.close()
    return

BS2(path)

b=pd.read_csv(path,header=None,sep='\s+')
#b=b[b[0]%1==0]
b.columns=['phi','rmp','rbs']


phi,f=np.mgrid[0:181:1,0:1:.1]
phif=np.array([phi.flatten(),f.flatten()]).T
c=pd.DataFrame(phif)
c.columns=['phi','f']
d=pd.merge(c,b,on='phi')
d['r']=d.rmp+d.f*(d.rbs-d.rmp)
d['x']=d.r*np.cos(d.phi*np.pi/180)
d['y']=d.r*np.sin(d.phi*np.pi/180)
d['ni']=d.apply(lambda x:n(x.x,x.y)[0],axis=1)
d['V']=d.apply(lambda x:v(x.x,x.y)[0],axis=1)
d['Ti']=1+64/3*(1-d.V**2)
path2='/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa2.txt'
d.to_csv(path2,float_format='%.4f',index=False)

a=pd.read_csv(path2)
#a=a.sort_values(by=['phi'])
phi=np.arange(0,181,1)
m=np.arange(0,10,1)
print(np.shape(a))

print(np.shape(a)[0]//10)
n,T,V=np.zeros([3,np.shape(a)[0]//10-1,10])
for k in range(np.shape(a)[0]//10-1):
    for l in range(len(m)):
        cur=a.iloc[10*k+l]
#        print(cur)
        n[k,l]=cur.ni
        T[k,l]=cur.Ti
        V[k,l]=cur.V
np.savetxt('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/Msh_n' ,n,fmt='%.3f')
np.savetxt('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/Msh_T' ,T,fmt='%.3f')
np.savetxt('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/Msh_V' ,V,fmt='%.3f')



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
from datetime import datetime,timedelta
from matplotlib.dates import DateFormatter,AutoDateLocator
from .sc_data import main as sc_data_read

def rolling_ave(tt,vv,tave,dt,avetype='c'):
  "calculate centered average and previous value average                        \
   tt   : time series in seconds						\
   vv   : data to be averaged							\
   tave : time series of moving average						\
   dt   : time duration of the moving average in seconds 			\
   avetype : c for centered average, p for averaging previous values 		"

  v1 = np.zeros(len(tave))
  n1 = np.zeros(len(tave))
  if   avetype=='c':		# centered average
   for i in range(len(vv)):
#    print(vv[i])
    if not np.isnan(vv[i]):
     for j in range(len(tave)):
#      print(type(tt[i]),type(tave[j]),type((dt)))
#      print(i,tt[i],tave[j])
      if tt[i]>=tave[j]-np.timedelta64(int(dt/2),'s') and tt[i] < tave[j]+np.timedelta64(int(dt/2),'s') :
        v1[j]=v1[j]+vv[i]
        n1[j]=n1[j]+1
  elif avetype=='p':		# average of previous values
   for i in range(len(vv)):
    if not np.isnan(vv[i]):
     for j in range(len(tave)):
      if j<len(tave)-1:
        if tt[i]>=tave[j] and tt[i] < tave[j+1] :
          v1[j+1]=v1[j+1]+vv[i]
          n1[j+1]=n1[j+1]+1
  vave = [ v1[i]/n1[i] for i in range(len(v1)) ]
  return vave

def main(argv):
    run=argv[0]
    ts=argv[1]
    te=argv[2]
    sc=argv[3]
    
    sc_data_read(run,ts,te,sc)

    fps1=f'{run}/{run}.Msh_model.png'
    fmsh=f'{run}/Msh_MHD_out.txt'
    fvel=f'{run}/Msh_Soucek_out.txt'
    fmag=f'{run}/Msh_Romashets_out.txt'
    fsp=f'{run}/Msh_Spreiter_out.txt'
    foff=f'{run}/Msh_offset.txt'
#    print(run,sc)

    if sc=='cluster4':
        
#        print('test')
        fc1=f'{run}/c4_fgm.lst'
        fc2=f'{run}/c4_cis.lst'
        title=f'{run} comparison of {sc} data with Mshpy results'
        compare_CLMHD(fps1,fc1,fc2,fmsh,fvel,fmag,fsp,title,foff=foff)

def compare_CLMHD(fps,fc1,fc2,fmsh,fvel,fmag,fsp,title,avt=60,foff=0):
    Msh=pd.read_csv(fmsh,parse_dates=['time'])
    vel=pd.read_csv(fvel,parse_dates=['time'])
    mag=pd.read_csv(fmag,parse_dates=['time'])
    spr=pd.read_csv(fsp,parse_dates=['time'])

    fgm=pd.read_csv(fc1,parse_dates=['time'])
    cis=pd.read_csv(fc2,parse_dates=['time'])

    tave=np.arange(Msh.time.iloc[0],Msh.time.iloc[-1],timedelta(seconds=avt))
    ave_df=pd.DataFrame({
        'bxav':rolling_ave(fgm.time,fgm.Bx,tave,avt),
        'byav':rolling_ave(fgm.time,fgm.By,tave,avt),
        'bzav':rolling_ave(fgm.time,fgm.Bz,tave,avt),
        'n1av':rolling_ave(cis.time,cis.n,tave,avt),
        'vxav':rolling_ave(cis.time,cis.Vx,tave,avt),
        'vyav':rolling_ave(cis.time,cis.Vy,tave,avt),
        'vzav':rolling_ave(cis.time,cis.Vz,tave,avt),
        'tsav':rolling_ave(cis.time,cis.Tev,tave,avt)
    })
    ave_df['btav']=np.sqrt(ave_df.bxav**2+ave_df.byav**2+ave_df.bzav**2)
    ave_df['vtav']=np.sqrt(ave_df.vxav**2+ave_df.vyav**2+ave_df.vzav**2)

    mag['Bt']=np.sqrt(mag.Bx**2+mag.By**2+mag.Bz**2)
    vel['vt']=np.sqrt(vel.vx**2+vel.vy**2+vel.vz**2)
    Msh['Bt']=np.sqrt(Msh.Bx**2+Msh.By**2+Msh.Bz**2)
    Msh['Vt']=np.sqrt(Msh.Vx**2+Msh.Vy**2+Msh.Vz**2)

    fig,ax=plt.subplots(10,1,sharex=True,figsize=(6.4,10))
    plt.subplots_adjust(hspace=0)
    
    ax[0].plot(tave,ave_df.bxav,'k-',label='Cluster')
    ax[0].plot(Msh.time,Msh.Bx,'g-',label='MHD-based')
    ax[0].plot(mag.time,mag.Bx,'b-',label='Romashets')
    ax[0].legend()

    ax[1].plot(tave,ave_df.byav,'k-',label='Cluster')
    ax[1].plot(Msh.time,Msh.By,'g-',label='MHD-based')
    ax[1].plot(mag.time,mag.By,'b-',label='Romashets')

    ax[2].plot(tave,ave_df.bzav,'k-',label='Cluster')
    ax[2].plot(Msh.time,Msh.Bz,'g-',label='MHD-based')
    ax[2].plot(mag.time,mag.Bz,'b-',label='Romashets')

    ax[3].plot(tave,ave_df.btav,'k-',label='Cluster')
    ax[3].plot(Msh.time,Msh.Bt,'g-',label='MHD-based')
    ax[3].plot(mag.time,mag.Bt,'b-',label='Romashets')

    ax[4].plot(tave,ave_df.vxav,'k-',label='Cluster')
    ax[4].plot(Msh.time,Msh.Vx,'g-',label='MHD-based')
    ax[4].plot(vel.time,vel.vx,'r-',label='Soucek')
    ax[4].legend()

    ax[5].plot(tave,ave_df.vyav,'k-',label='Cluster')
    ax[5].plot(Msh.time,Msh.Vy,'g-',label='MHD-based')
    ax[5].plot(vel.time,vel.vy,'r-',label='Soucek')

    ax[6].plot(tave,ave_df.vzav,'k-',label='Cluster')
    ax[6].plot(Msh.time,Msh.Vz,'g-',label='MHD-based')
    ax[6].plot(vel.time,vel.vz,'r-',label='Soucek')

    ax[7].plot(tave,ave_df.vtav,'k-',label='Cluster')
    ax[7].plot(Msh.time,Msh.Vt,'g-',label='MHD-based')
    ax[7].plot(vel.time,vel.vt,'r-',label='Soucek')
    ax[7].plot(spr.time,spr.V,'m-',label='Spreiter')

    ax[8].plot(tave,ave_df.n1av,'k-',label='Cluster')
    ax[8].plot(Msh.time,Msh.n,'g-',label='MHD-based')
    ax[8].plot(spr.time,spr.n,'m-',label='Spreiter')
    ax[8].legend()

    ax[9].plot(tave,ave_df.tsav,'k-',label='Cluster')
    ax[9].plot(Msh.time,Msh.Tev,'g-',label='MHD-based')
    ax[9].plot(spr.time,spr.Tev,'m-',label='Spreiter')
    ax[9].set_xlabel(f'Time starting from {(tave[0]).astype(datetime).strftime("%b %d %H:%MUT")}')

    ylab= [ 'Bx [nT]',            \
            'By [nT]',            \
            'Bz [nT]',            \
            '|B| [nT]',            \
            'Vx [km/s]',          \
            'Vy [km/s]',          \
            'Vz [km/s]',          \
            '|V| [km/s]',          \
            'N [cm$^{-3}$]',      \
            'T [eV]'              ]

    for i in range(10):
        ax[i].grid()
        ax[i].set_ylabel(ylab[i])
        ax[i].xaxis.set_major_formatter(DateFormatter("%H:%M"))   

    if foff!=0:
        with open(foff) as f:
            for line in f:
                mpdo,bsdo = line.split()
                mpdo=float(mpdo)
                bsdo=float(bsdo)
    #        print(mpdo,bsdo)
        label=[]
        annotate=f'MP adjustment = {mpdo}R$_E$\nBS adjustment = {bsdo}R$_E$'
        print(annotate)
    
    anloc=(.1,.03)
    plt.annotate(annotate,xy=anloc, xycoords='figure fraction')
    print("Save",fps)
    plt.savefig(fps)
    plt.close()
    
    return

if __name__=='__main__':
    main(sys.argv[1:])


    


import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv

external=argv[1]
run=argv[2]

for i in range(0,2):
    a=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_data_0.1.'+str(7200+7200*i).zfill(6),header=None,sep='\s+',
    names=['the','phi','m','rad','x','y','z','bx1','by1','bz1','vx1','vy1','vz1','rr1','pp1','ex1','ey1','ez1','xjx1','xjy1','xjz1','resis1']).drop_duplicates()

    a=a.sort_values(by=['the','phi'])
    the=np.arange(-90,91,1)
    phi=np.arange(-90,91,1)
    m=np.arange(0,11,1)
    print(np.shape(a))

    Bx,By,Bz,Vx,Vy,Vz,n,pp=np.zeros([8,181,181,11])
    for j in range(len(the)):
        for k in range(len(phi)):
            for l in range(len(m)):
                cur=a.iloc[(j*181+k)*11+l]
                Bx[j,k,l]=cur.bx1
                By[j,k,l]=cur.by1
                Bz[j,k,l]=cur.bz1
                Vx[j,k,l]=cur.vx1
                Vy[j,k,l]=cur.vy1
                Vz[j,k,l]=cur.vz1
                n[j,k,l]=cur.rr1
                pp[j,k,l]=cur.pp1
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Bx_%d' % i,Bx)
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_By_%d' % i,By)
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Bz_%d' % i,Bz)
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vx_%d' % i,Vx)
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vy_%d' % i,Vy)
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vz_%d' % i,Vz)
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_n_%d' % i,n)
    np.save('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_pp_%d' % i,pp)



#!/usr/bin/python
#
# get sc orbit file to obtain MHD values along spacecraft orbit


import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")
sys.path.append(HOME+"/svn/hkc-project/trunk/src/swdata/")

import locale
#from jrmod01            import *
from numpy              import arange
from openggcm_info      import mhdtime,mhdt2hrmi
from swdata		import interpolate_data

def ldaf(filename):
    with open(filename) as myFile:
        for num, line in enumerate(myFile, 1):
            if line.startswith('dd'):
                lda=num
#                print ('found at line:', num)
    return lda


def read_THFGM(fi,vtype='string'):
  "read THEMIS FGM data downloaded from SPDweb"

  Err = '-1.00000E+31'
  re  = 6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  date=[]
  time=[]
  bx=[];by=[];bz=[]
#  xl=[];yl=[];zl=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err and w[4]!=Err  :

      date.append(w[0])
      time.append(w[1])
      bx1=locale.format('%f',locale.atof(w[2]))
      by1=locale.format('%f',locale.atof(w[3]))
      bz1=locale.format('%f',locale.atof(w[4]))
      bx.append(bx1)
      by.append(by1)
      bz.append(bz1)
#      x1=locale.format('%f',locale.atof(w[5])/re)
#      y1=locale.format('%f',locale.atof(w[6])/re)
#      z1=locale.format('%f',locale.atof(w[7])/re)
#      xl.append(x1)
#      yl.append(y1)
#      zl.append(z1)

  if vtype=='real':
    bx = [locale.atof(i) for i in bx ]
    by = [locale.atof(i) for i in by ]
    bz = [locale.atof(i) for i in bz ]
#    xl = [locale.atof(i) for i in xl  ]
#    yl = [locale.atof(i) for i in yl  ]
#    zl = [locale.atof(i) for i in zl  ]

  return date,time,bx,by,bz

def read_THorbit(fi,lda,vtype='string'):
  "read THEMIS orbit data downloaded from SPDweb"

  Err = '-1.00000E+31'
  re  = 6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  date=[]
  time=[]
  bx=[];by=[];bz=[]
#  xl=[];yl=[];zl=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err and w[4]!=Err  :

      date.append(w[0])
      time.append(w[1])
      bx1=locale.format('%f',locale.atof(w[2]))
      by1=locale.format('%f',locale.atof(w[3]))
      bz1=locale.format('%f',locale.atof(w[4]))
      bx.append(bx1)
      by.append(by1)
      bz.append(bz1)
#      x1=locale.format('%f',locale.atof(w[5])/re)
#      y1=locale.format('%f',locale.atof(w[6])/re)
#      z1=locale.format('%f',locale.atof(w[7])/re)
#      xl.append(x1)
#      yl.append(y1)
#      zl.append(z1)

  if vtype=='real':
    bx = [locale.atof(i) for i in bx ]
    by = [locale.atof(i) for i in by ]
    bz = [locale.atof(i) for i in bz ]
#    xl = [locale.atof(i) for i in xl  ]
#    yl = [locale.atof(i) for i in yl  ]
#    zl = [locale.atof(i) for i in zl  ]

  return date,time,bx,by,bz


def read_THESA(fi,vtype='string'):
  "read THEMIS ESA data"

  Err = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[];time=[];nsw=[];tk=[];teV=[];nhp=[];thp=[];php=[];
  vswx=[];vswy=[];vswz=[];vhpx=[];vhpy=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err  and w[4]!=Err  and  \
       w[5]!=Err and w[6]!=Err :

      date.append(w[0])
      time.append(w[1])
      teV.append(w[3])
#      teV.append(locale.format('%f',locale.atof(w[2])/11600.))  # convert K to eV
      tk.append(w[3])
      nsw.append(w[2])
      vswx.append(w[4])
      vswy.append(w[5])
      vswz.append(w[6])

  if vtype=='real':
    nsw  = [locale.atof(i) for i in nsw ]
    tk   = [locale.atof(i) for i in tk  ]
    teV  = [locale.atof(i) for i in teV ]
    vswx = [locale.atof(i) for i in vswx ]
    vswy = [locale.atof(i) for i in vswy ]
    vswz = [locale.atof(i) for i in vswz ]

  return date,time,nsw,teV,vswx,vswy,vswz

def read_THESAr(fi,vtype='string'):
  "read THEMIS ESA data- number density and temperature from reduced data"

  lda=ldaf(fi)


  Err = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()


  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[];time=[];nsw=[];tk=[];teV=[];nhp=[];thp=[];php=[];
  vswx=[];vswy=[];vswz=[];vhpx=[];vhpy=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err:

      date.append(w[0])
      time.append(w[1])
      tk.append(w[3])
      nsw.append(w[2])

  if vtype=='real':
    nsw  = [locale.atof(i) for i in nsw ]
    tk   = [locale.atof(i) for i in tk  ]

  return date,time,nsw,tk



def main(argv):

  sc=argv[0]                    # satellite info
  f1=argv[1]                    #MFI data: date time bxgse bygse bzgse
  f2=argv[2]                    #SWE data: date time vth rr xgse ygse zgse vxgse vygse vzgse
  t1=argv[3]                    #starttime for interpolation
  t2=argv[4]                    #endtime   for interpolation
  t3=argv[5]                    #starttime for MHD simulation
  t4=argv[6]                    #endtime   for MHD simulation
  av=argv[7]                    #time interval of interpolation in sec

  lda=argv[8].split(':')        #the line where data starts
  lda1=locale.atoi(lda[0])-1
  lda2=locale.atoi(lda[1])-1

  print(lda,lda1,lda2)

  create_GEdata(sc,f1,f2,lda1,lda2,t1,t2,t3,t4,av)


  return


if __name__=='__main__':
   main(sys.argv[1:])



# get 2D data from f3df
# read bx,by,bz(nT),vx,vy,vz(km/s),rr(/cm3),pp(pPa) from Msh model

# fout : output
# fgrid: grid file
# f3df : 3df file
# l1   : values to read  ex. bx,by,bz
# pn   : plane xy,yz,xz
# l2   : set plane [re]  ex. y=10.0 means a plane at y=10re

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
from Msh_Nstep_test_n import main as mhd
from scipy import constants

rrio1=5.
rrio2=7.
pie=3.141592654

xi=float(argv[2])
xf=float(argv[4])
yi=float(argv[6])
yf=float(argv[8])
ds=float(argv[10])
lp=float(argv[12])

pn=argv[14]
f_out=argv[16]
f_out2=f_out+'_temp'
print('xi,xf,yi,yf,ds,pn,lp',xi,xf,yi,yf,ds,pn,lp)

nnx=int((xf-xi)/ds+1)
nny=int((yf-yi)/ds+1)


f=open(f_out,'w')
f.write('%d %d\n' %(nnx,nny))

x=[]
y=[]
z=[]
for i in range(1,nny+1):
    for j in range(1,nnx+1):

        if pn=='xr':
            xr=xi+(j-1)*ds
            yr=0.
            zr=yi+(i-1)*ds
            xx=xr
            yy=yr*np.cos(lp*np.pi/180.)-zr*np.sin(lp*np.pi/180.)
            zz=yr*np.sin(lp*np.pi/180.)+zr*np.cos(lp*np.pi/180.)
        elif pn=='zr':
            xr=xi+(j-1)*ds
            yr=0.
            zr=yi+(i-1)*ds
            xx=xr*np.cos(lp*np.pi/180.)-yr*np.sin(lp*np.pi/180.)
            yy=xr*np.sin(lp*np.pi/180.)+yr*np.cos(lp*np.pi/180.)
            zz=zr
        elif pn=='xz':
            xx=xi+(j-1)*ds
            yy=lp
            zz=yi+(i-1)*ds
        elif pn=='xy':
            xx=xi+(j-1)*ds
            yy=yi+(i-1)*ds
            zz=lp
        elif pn=='yz':
            xx=lp
            yy=xi+(j-1)*ds
            zz=yi+(i-1)*ds
        x.append(xx)
        y.append(yy)
        z.append(zz)

xyz=np.array(list(zip(x,y,z))).astype('float64')
print(xyz)
f_sw='SW_cond_test.txt'
mhd(xyz,f_sw,f_out2)
print('Model value extracted')

Msh=pd.read_csv(f_out2)

for i in range(1,nny+1):
    for j in range(1,nnx+1):
        temp=Msh.iloc[j-1+(i-1)*nnx]
        bx1=temp.Bx
        by1=temp.By
        bz1=temp.Bz
        vx1=temp.Vx
        vy1=temp.Vy
        vz1=temp.Vz
        rr1=temp.n
        tk1=temp.Tev
        pp1=11600e18*constants.k*rr1*tk1
        xx=x[j-1+(i-1)*nnx]
        yy=y[j-1+(i-1)*nnx]
        zz=z[j-1+(i-1)*nnx]

        if pn=='xr' or pn=='zr':
            f.write('%d %d %f %f %f %f %f %f %f %f %f %f %f nan nan nan nan nan nan %f %f %f nan\n'%(j,i,xx,yy,zz,bx1,by1,bz1,vx1,vy1,vz1,rr1,pp1,xr,yr,zr))
        else:
            f.write('%d %d %f %f %f %f %f %f %f %f %f %f %f nan nan nan nan nan nan nan\n'%(j,i,xx,yy,zz,bx1,by1,bz1,vx1,vy1,vz1,rr1,pp1))
    if i%10==0: print(i)
f.close()



#Compare 10^5K run and 10^4K run temperature along radial directions

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd

f1='/Volumes/easystore/openggcm_run/Msh_Nstep/Msh_Nstep.3D.002400'
f2='/Volumes/Elements/openggcm_run/Msh_Nstep_003/Msh_Nstep_003.3D.002400'

a=pd.read_csv(f1,names=['i','the','j','radi','x1','y1','z1','rr1','k','phi','rr1','pp1'],sep='\s+',skiprows=1)
b=pd.read_csv(f2,names=['i','the','j','radi','x1','y1','z1','rr1','k','phi','rr1','pp1'],sep='\s+',skiprows=1)
a=a[a.the==0]
b=b[b.the==0]
a['T1']=72429*a.pp1/a.rr1/11600
b['T1']=72429*b.pp1/b.rr1/11600



fig,axes=plt.subplots(1,5,figsize=(15,6))

phil=[-90,-45,0,45,90]
for i in range(5):
    tempa=a[a.phi==phil[i]].sort_values(by=['radi'])
    tempb=b[b.phi==phil[i]].sort_values(by=['radi'])
    axes[i].plot(tempa.radi,tempa.T1,label='$T_{sw}=10^5K$')
    axes[i].plot(tempb.radi,tempb.T1,label='$T_{sw}=10^4K$')
    axes[i].legend()
    axes[i].set_xlabel('r [Re]')
    axes[i].set_ylabel('T [eV]')
    axes[i].set_title('$\phi=$'+str(phil[i])+'$^\circ$')
    axes[i].grid()

plt.tight_layout()
plt.savefig('Msh_temp_test.png')



#!/Users/jjung11/anaconda3/bin/python
# Calculate input for plot_rates.py
# Written by J.Jung


import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
from sympy.solvers import solve
from sympy import Symbol,nsolve
from sympy import sqrt,log,tanh
from scipy.interpolate import NearestNDInterpolator as NNDI
from scipy.interpolate import LinearNDInterpolator as LNDI
from scipy.interpolate import RegularGridInterpolator as RGI
from scipy.spatial.transform import Rotation as R
import glob
import datetime

# Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

def bsdf(D,f,pts):

    sol0=a44
    bsd=[]
#    print(pts[4])
    dx=0.1*np.cos(pts[:,4])*np.cos(pts[:,5])
    dy=0.1*np.cos(pts[:,4])*np.sin(pts[:,5])
    dz=0.1*np.sin(pts[:,4])

    for dxi,dyi,dzi in zip(dx,dy,dz):
        sol=sol0
#        print(sol,sol0)
        x=100*dxi
        y=100*dyi
        z=100*dzi
        while sol*sol0>0:
            x+=dxi
            y+=dyi
            z+=dzi
            sol=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
            #print(sol)
        dist=np.sqrt(x**2+y**2+z**2)
    bsd.append(dist)
    return bsd

def bsdf_Jelinek(a14,a44,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    cos=np.cos(theta)
    sin=np.sin(theta)
    r=(-a14*cos+np.sqrt(a14**2*cos**2-4*a44*sin**2))/(2*sin**2)
    r0=-a44/a14
    r[np.isnan(r)]=r0
    return r



def mpdf(r0,alpha,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    R = r0[0]*(2/(1+np.cos(theta)))**alpha[0]
    return R

def genf(fun,n1,n2,xyz,f,f2):
    outl=[]
    for i in xyz:
        if i[2]<0.1 or i[2]>0.9:
            out=np.nan
        else:
            if f2==-1:
                out1=fun[n1+7](i)[0]
                out2=fun[n2+7](i)[0]
                out=(out1*(1-f)+out2*f)
            elif f2==2:
                out1=fun[n1](i)[0]
                out2=fun[n2](i)[0]
                out=(out1*(1-f)+out2*f)
            else:
                out1=fun[n1](i)[0]
                out2=fun[n2](i)[0]
                out=(out1*(1-f)+out2*f)
                out1=fun[n1+7](i)[0]
                out2=fun[n2+7](i)[0]
                outs=(out1*(1-f)+out2*f)
                out=f2*out+(1-f2)*outs
        outl.append(out)
    outl=np.array(outl)
    return outl

nfl=[]
Tfl=[]
Vxfl=[]
Vyfl=[]
Vzfl=[]
the=np.arange(-90,91,1)*np.pi/180
phi=np.arange(-90,91,1)*np.pi/180
af=np.arange(0,11,1)/10

def OMNIread2(f):

    BErr='9999.99'          # err of B field
    VErr='99999.9'          # err of Velocity
    NErr='999.99'               # err of Number density

    time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            vx.append(float(w[7]))
            vy.append(float(w[8]))
            vz.append(float(w[9]))
            n.append(float(w[10]))
            Pd.append(float(w[11]))
            Ma.append(float(w[12]))
            Mm.append(float(w[13]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    vx=np.array(vx)
    vy=np.array(vy)
    vz=np.array(vz)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    Mm=np.array(Mm)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    V=np.sqrt(vx**2+vy**2+vz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw})
    return omni

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
    ptsnew[:,4] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew



path=sys.argv[2]
y1=float(sys.argv[4])
y2=float(sys.argv[6])
dy=float(sys.argv[8])
x1=float(sys.argv[10])
x2=float(sys.argv[12])
dx=float(sys.argv[14])
fout=sys.argv[16]
print(path,x1,x2,dx,y1,y2,dy,fout)

for i in range(0,7):
    rr1=np.load(path+'/cn/Msh_n_%d.npy'%i)
    pp1=np.load(path+'/cn/Msh_pp_%d.npy'%i)
    T1=72429*pp1/rr1/11600
    vx1=np.load(path+'/cn/Msh_Vx_%d.npy'%i)
    vy1=np.load(path+'/cn/Msh_Vy_%d.npy'%i)
    vz1=np.load(path+'/cn/Msh_Vz_%d.npy'%i)

    nf=RGI((the,phi,af),rr1,bounds_error=False)
    Tf=RGI((the,phi,af),T1,bounds_error=False)
    Vxf=RGI((the,phi,af),vx1,bounds_error=False)
    Vyf=RGI((the,phi,af),vy1,bounds_error=False)
    Vzf=RGI((the,phi,af),vz1,bounds_error=False)
    nfl.append(nf)
    Tfl.append(Tf)
    Vxfl.append(Vxf)
    Vyfl.append(Vyf)
    Vzfl.append(Vzf)

for i in range(0,7):
    rr1=np.load(path+'/cs/Msh_n_%d.npy'%i)
    pp1=np.load(path+'/cs/Msh_pp_%d.npy'%i)
    T1=72429*pp1/rr1/11600
    vx1=np.load(path+'/cs/Msh_Vx_%d.npy'%i)
    vy1=np.load(path+'/cs/Msh_Vy_%d.npy'%i)
    vz1=np.load(path+'/cs/Msh_Vz_%d.npy'%i)

    nf=RGI((the,phi,af),rr1,bounds_error=False)
    Tf=RGI((the,phi,af),T1,bounds_error=False)
    Vxf=RGI((the,phi,af),vx1,bounds_error=False)
    Vyf=RGI((the,phi,af),vy1,bounds_error=False)
    Vzf=RGI((the,phi,af),vz1,bounds_error=False)
    nfl.append(nf)
    Tfl.append(Tf)
    Vxfl.append(Vxf)
    Vyfl.append(Vyf)
    Vzfl.append(Vzf)



nh0=25
drad=0.2*6372*1e3
ri=2.1
str1=4*np.pi
crs=6e-16




f_sw="SW_cond_test.txt"
test=open(f_sw,"w")
test.write("1967 1 0 00 0 0 5 -400 0 0 10 2 10 6\n")
#Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
test.close()
omni=OMNIread2(f_sw)
D=0.937*(0.846+0.042*omni.B)
f=(91.55/(omni.n*omni.V**2)**(1/6)*(1+D*((5/3-1)*omni.Ma**2+2)/((5/3+1)*(omni.Ma**2-1)))/R0)[0]
r0=(10.22+1.29*np.tanh(0.184*(omni.Bz+8.14)))*omni.Pd**(-1/6.6)
alpha=(0.58-0.007*omni.Bz)*(1+0.024*np.log(omni.Pd))
mp0=mpdf(r0,alpha,np.array([[1,0,0]]))
bs0=bsdf(D,f,np.array([[0,0,0,1,0,0]]))
print('Shue/Jerab',mp0,bs0)
R0=15.02
lam=1.17
epsilon=6.55
a14=(4*R0/lam**2*omni.Pd**(-1/epsilon))[0]
a44=(-1/4*lam**2*a14**2)
bs0_j=bsdf_Jelinek(a14,a44,np.array([[1,0,0]]))
print('Shue/Jelinek',mp0,bs0_j)


#bsdfv=np.vectorize(bsdf)
if (omni.n>35).any():print('Warning: Solar wind density goes above 35cm^-3, in which our model was not validated.')
if (omni.B>15).any():print('Warning: IMF magnitude goes above 15cm^-3, in which our model shows discrepancy with THEMIS data.')

conditions=[omni.n<=1,(1<omni.n) & (omni.n<=5),(5<omni.n) & (omni.n<10),(10<=omni.n) & (omni.n<15),(15<=omni.n) & (omni.n<20),(20<=omni.n) & (omni.n<25),(25<=omni.n) & (omni.n<30),(30<=omni.n) & (omni.n<35)]
choices1=[0,0,1,2,3,4,5,6]
choices2=[0,1,2,3,4,5,6,6]
n1=np.select(conditions,choices1)[0]
n2=np.select(conditions,choices2)[0]
f1=np.where(n2==1,(omni.n-1)/4,omni.n/5-n1)[0]
conditions2=[omni.Bz<=-4,(-4<omni.Bz) & (omni.Bz<4), 4<=omni.Bz]
choices3=[-1,(omni.Bz+4)/8,2]
f2=np.select(conditions2,choices3)[0]
print('f1,f2',f1,f2)


fo=open(fout,'w+')
#fout2='test.txt'
#fo2=open(fout2,'w+')
fo.write('%d %d\n' % (((x2-x1)/dx+1),((y2-y1)/dy+1)))
yl=np.arange(y1,y2+dy,dy)
xl=np.arange(x1,x2+dx,dx)
for i,y in enumerate(yl):
    for j,x in enumerate(xl):
        rate=0
        ro=np.sqrt(x**2+y**2)
        if ro<ri:
            rate=9999
#                print(x,y,z)
        else:
            xyz_0=np.array([[x,y,0,ro]])

            rm=R.from_euler('z',-4,degrees=True)
            xyz_np=(rm.as_matrix()@xyz_0[:,:-1].T).T
            #xyz_np=los[:,:-1]

    #        print('Finding MP nodes')
            mpd=np.array(mpdf(r0,alpha,xyz_np))

     #       print('Finding BS nodes')
            #print(xyz_np)
            pts=appendSpherical_np(xyz_np)
            #bsd=np.array(bsdf(D,f,pts))
            bsd=np.array(bsdf_Jelinek(a14,a44,xyz_np))

            f_msh=(pts[:,3]-mpd)/(bsd-mpd)
            pts=np.hstack((pts,f_msh.reshape(pts.shape[0],1)))
        #    print(f_msh)

            input=pts[:,4:]
    #        print(np.shape(input))

            #print(phi,np.shape(nfl),np.shape(n1),np.shape(n2),np.shape(input),np.shape(f),np.shape(f2))
            n=genf(nfl,n1,n2,input,f1,f2)
            T=genf(Tfl,n1,n2,input,f1,f2)
            Vx=genf(Vxfl,n1,n2,input,f1,f2)
            Vy=genf(Vyfl,n1,n2,input,f1,f2)
            Vz=genf(Vzfl,n1,n2,input,f1,f2)
    #        V_0=np.vstack([Vx,Vy,Vz]).T
    #        V=(rm.as_matrix().T@V_0.T).T
            vth=np.sqrt(3*1.38e-23*T/1.6727e-27)*100
            vbu=np.sqrt(Vx**2+Vy**2+Vz**2)*1e5
            vrel=np.sqrt(vth**2+vbu**2)
            nn=nh0*(10./xyz_0[:,-1])**3
            xray=n*vrel*nn*drad*100*crs #eV/cm^3/s
            rate=xray/str1 #cV/cm^3/s/sr



        fo.write('%d %d %f %f %e\n' % (j,i,x,y,rate))
#        print(x,y,f_msh,rate)
    if i%10==0: print('%.3f / %f' %(y,y2-y1))
fo.close()
#fo2.close()



import numpy as np
import matplotlib.pyplot as plt
from sunpy.net import Fido, attrs as a
from sunpy.timeseries import TimeSeries
from sunpy.time import TimeRange
from sys import argv
from datetime import datetime, timedelta

def main(run,ts,te,sc):

#    t0=datetime.strptime(ts,'%Y-%m-%dT%H:%M')
#    t1=t0+timedelta(days=1)

    if sc=='cluster4':
        dataset=a.cdaweb.Dataset('C4_CP_FGM_SPIN')
        result=Fido.search(a.Time(ts,te),dataset)
        downloaded_files=Fido.fetch(result[0])
        fgm=TimeSeries(downloaded_files,concatenate=True)
        tr=TimeRange(ts,te)
        fgm_trunc=fgm.truncate(tr)
        df_fgm=fgm_trunc.to_dataframe()
        df_fgm2=df_fgm.iloc[:,1:4]
        df_fgm2.columns=['Bx','By','Bz']
        df_fgm2.index.names=['time']
        df_fgm2.to_csv(f'{run}/c4_fgm.lst')


        dataset=a.cdaweb.Dataset('C4_PP_CIS')
        result=Fido.search(a.Time(ts,te),dataset)
        downloaded_files=Fido.fetch(result[0])
        cis=TimeSeries(downloaded_files,concatenate=True)
        tr=TimeRange(ts,te)
        cis_trunc=cis.truncate(tr)
        df_cis=cis_trunc.to_dataframe()
        df_cis2=df_cis.iloc[:,[4,11,12,16,17,18]].copy()
        df_cis2.columns=['n','T_par','T_perp','Vx','Vy','Vz']
        df_cis2['Tev']=(df_cis2.T_par+df_cis2.T_perp)*1e6/(2*11600)
        df_cis3=df_cis2.drop(columns=['T_par','T_perp'])
        df_cis3.index.names=['time']
        df_cis3.to_csv(f'{run}/c4_cis.lst')
        return

if __name__=='__main__':
    main(*argv[1:])



import numpy as np
import matplotlib.pyplot as plt
from sunpy.net import Fido, attrs as a
from sunpy.timeseries import TimeSeries
from sunpy.time import TimeRange
from datetime import datetime,timedelta
from sys import argv

def main(run,ts,te):
#    t0=datetime.strptime(ts,'%Y-%m-%dT%H:%M')
#    t1=t0+timedelta(days=1)

    dataset=a.cdaweb.Dataset('OMNI_HRO2_1MIN')
    result=Fido.search(a.Time(ts,te),dataset)
    downloaded_files=Fido.fetch(result[0])

    omni=TimeSeries(downloaded_files,concatenate=True)
    tr=TimeRange(ts,te)
    omni_trunc=omni.truncate(tr)

    # Convert TimeSeries to Pandas DataFrame
    df = omni_trunc.to_dataframe()
    df2=df[['YR','Day','HR','Minute','BX_GSE','BY_GSE','BZ_GSE','Vx','Vy','Vz','proton_density', 'Pressure', 'Mach_num', 'Mgs_mach_num']]
    df2.to_csv('test/omni.lst',index=False,header=None,sep='\t',na_rep='nan')

    return
#fig,ax=plt.subplots()
#plt.ion()
#omni_trunc.plot(y=['BZ_GSE'],axes=ax)
#plt.show()

if __name__=='__main__':
    main(*argv[1:])